@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2022 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -73,8 +73,9 @@ EVT_MENU(ID_FILE_EXIT, GOFrame::OnExit)
 EVT_MENU(ID_FILE_RELOAD, GOFrame::OnReload)
 EVT_MENU(ID_FILE_REVERT, GOFrame::OnRevert)
 EVT_MENU(ID_FILE_PROPERTIES, GOFrame::OnProperties)
-EVT_MENU(ID_FILE_IMPORT_SETTINGS, GOFrame::OnImportSettings)
 EVT_MENU(ID_FILE_IMPORT_COMBINATIONS, GOFrame::OnImportCombinations)
+EVT_MENU(ID_FILE_EXPORT_COMBINATIONS, GOFrame::OnExportCombinations)
+EVT_MENU(ID_FILE_IMPORT_SETTINGS, GOFrame::OnImportSettings)
 EVT_MENU(ID_FILE_EXPORT, GOFrame::OnExport)
 EVT_MENU(ID_FILE_CACHE, GOFrame::OnCache)
 EVT_MENU(ID_FILE_CACHE_DELETE, GOFrame::OnCacheDelete)
@@ -201,21 +202,23 @@ GOFrame::GOFrame(
   m_file_menu->Append(
     ID_FILE_REVERT, _("Reset to &Defaults"), wxEmptyString, wxITEM_NORMAL);
   m_file_menu->AppendSeparator();
-  m_file_menu->Append(
-    ID_FILE_IMPORT_SETTINGS,
-    _("&Import Settings"),
-    wxEmptyString,
-    wxITEM_NORMAL);
   m_file_menu->Append(
     ID_FILE_IMPORT_COMBINATIONS,
     _("Import &Combinations"),
     wxEmptyString,
     wxITEM_NORMAL);
   m_file_menu->Append(
-    ID_FILE_EXPORT,
-    _("&Export Settings/Combinations"),
+    ID_FILE_EXPORT_COMBINATIONS,
+    _("Export &Combinations"),
     wxEmptyString,
     wxITEM_NORMAL);
+  m_file_menu->Append(
+    ID_FILE_IMPORT_SETTINGS,
+    _("&Import Settings"),
+    wxEmptyString,
+    wxITEM_NORMAL);
+  m_file_menu->Append(
+    ID_FILE_EXPORT, _("&Export Settings"), wxEmptyString, wxITEM_NORMAL);
   m_file_menu->AppendSeparator();
   m_file_menu->Append(
     ID_SETTINGS, wxT("&Settings..."), wxEmptyString, wxITEM_NORMAL);
@@ -317,8 +320,7 @@ GOFrame::GOFrame(
     wxDefaultSize,
     choices);
   m_ToolBar->AddControl(m_ReleaseLength);
-  m_Sound.GetEngine().SetReleaseLength(m_config.ReleaseLength());
-  UpdateReleaseLength();
+  UpdateReleaseLength(m_config.ReleaseLength());
 
   m_ToolBar->AddTool(
     ID_TRANSPOSE,
@@ -474,10 +476,16 @@ void GOFrame::SetPosSize(const GOLogicalRect &lRect) {
   }
 }
 
-void GOFrame::UpdateReleaseLength() {
-  unsigned n = m_config.ReleaseLength();
+void GOFrame::UpdateReleaseLength(unsigned releaseLength) {
+  GOOrganController *organController = GetOrganController();
+  int releaseLengthIndex = releaseLength / 50;
 
-  m_ReleaseLength->SetSelection(n / 50);
+  if (organController && organController->GetReleaseTail() != releaseLength)
+    organController->SetReleaseTail(releaseLength);
+  if (m_config.ReleaseLength() != releaseLength)
+    m_config.ReleaseLength(releaseLength);
+  if (m_ReleaseLength->GetSelection() != releaseLengthIndex)
+    m_ReleaseLength->SetSelection(releaseLengthIndex);
 }
 
 void GOFrame::UpdateVolumeControlWithSettings() {
@@ -540,6 +548,13 @@ void GOFrame::Init(wxString filename) {
   clean.Cleanup();
 }
 
+void GOFrame::AttachDetachOrganController(bool isToAttach) {
+  GOOrganController *pOrgan = GetOrganController();
+
+  if (pOrgan)
+    pOrgan->SetModificationListener(isToAttach ? this : nullptr);
+}
+
 bool GOFrame::CloseOrgan(bool isForce) {
   bool isClosed = true;
 
@@ -567,6 +582,7 @@ bool GOFrame::CloseOrgan(bool isForce) {
       GOMutexLocker m_locker(m_mutex, true);
 
       if (m_locker.IsLocked()) {
+        AttachDetachOrganController(false);
         delete m_doc;
         m_doc = NULL;
         UpdatePanelMenu();
@@ -577,17 +593,30 @@ bool GOFrame::CloseOrgan(bool isForce) {
   return isClosed;
 }
 
+bool GOFrame::LoadOrgan(const GOOrgan &organ, const wxString &cmb) {
+  bool retCode = false;
+
+  if (m_doc) {
+    GOProgressDialog dlg;
+
+    retCode = m_doc->LoadOrgan(&dlg, organ, cmb);
+    OnIsModifiedChanged(false);
+
+    // for reflecting model changes
+    AttachDetachOrganController(true);
+  }
+  return retCode;
+}
+
 void GOFrame::Open(const GOOrgan &organ) {
-  if (!CloseOrgan(false))
-    return;
-  GOMutexLocker m_locker(m_mutex, true);
-  if (!m_locker.IsLocked())
-    return;
-  GOProgressDialog dlg;
-  m_doc = new GODocument(this, &m_Sound);
-  m_doc->Load(&dlg, organ);
-  UpdateReleaseLength();
-  UpdatePanelMenu();
+  if (CloseOrgan(false)) {
+    GOMutexLocker m_locker(m_mutex, true);
+
+    if (m_locker.IsLocked()) {
+      m_doc = new GODocument(this, &m_Sound);
+      LoadOrgan(organ);
+    }
+  }
 }
 
 GOOrganController *GOFrame::GetOrganController() const {
@@ -602,6 +631,14 @@ void GOFrame::OnPanel(wxCommandEvent &event) {
     m_doc->ShowPanel(no);
 }
 
+void GOFrame::OnIsModifiedChanged(bool modified) {
+  GOOrganController *organController = GetOrganController();
+
+  if (organController)
+    UpdateReleaseLength(organController->GetReleaseTail());
+  UpdatePanelMenu();
+}
+
 void GOFrame::UpdatePanelMenu() {
   GOOrganController *organController = GetOrganController();
   unsigned panelcount = (organController && organController->GetPanelCount())
@@ -746,7 +783,7 @@ void GOFrame::OnUpdateLoaded(wxUpdateUIEvent &event) {
   if (event.GetId() == ID_AUDIO_MEMSET)
     event.Check(
       organController && organController->GetSetter()
-      && organController->GetSetter()->IsSetterActive());
+      && organController->GetSetter()->GetState().m_IsActive);
   else if (event.GetId() == ID_ORGAN_EDIT)
     event.Check(m_doc && m_doc->WindowExists(GODocument::ORGAN_DIALOG, NULL));
   else if (event.GetId() == ID_MIDI_LIST)
@@ -845,6 +882,60 @@ void GOFrame::OnInstall(wxCommandEvent &event) {
     }
 }
 
+void GOFrame::OnImportCombinations(wxCommandEvent &event) {
+  GOOrganController *pOrganController = GetOrganController();
+
+  if (pOrganController) {
+    wxFileDialog dlg(
+      this,
+      _("Import Combinations"),
+      pOrganController->GetCombinationsDir(),
+      wxEmptyString,
+      _("Combinations files (*.yaml)|*.yaml|Settings files (*.cmb)|*.cmb"),
+      wxFD_OPEN | wxFD_FILE_MUST_EXIST);
+
+    if (dlg.ShowModal() == wxID_OK)
+      pOrganController->LoadCombination(dlg.GetPath());
+  }
+}
+
+void GOFrame::OnExportCombinations(wxCommandEvent &event) {
+  GOOrganController *pOrganController = GetOrganController();
+
+  if (pOrganController) {
+    const wxString organCmbDir = pOrganController->GetCombinationsDir();
+
+    wxFileName::Mkdir(organCmbDir, wxS_DIR_DEFAULT, wxPATH_MKDIR_FULL);
+
+    wxFileDialog dlg(
+      this,
+      _("Export Combinations"),
+      organCmbDir,
+      wxEmptyString,
+      _("Combination files (*.yaml)|*.yaml"),
+      wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
+
+    if (dlg.ShowModal() == wxID_OK) {
+      wxString exportedFilePath = dlg.GetPath();
+
+      if (!exportedFilePath.EndsWith(wxT(".yaml"), NULL))
+        exportedFilePath += wxT(".yaml");
+      const wxString errMsg
+        = pOrganController->ExportCombination(exportedFilePath);
+
+      if (!errMsg.IsEmpty())
+        GOMessageBox(
+          wxString::Format(
+            _("Failed to export combinations to '%s': %s"),
+            exportedFilePath,
+            errMsg),
+          _("Error"),
+          wxOK | wxICON_ERROR,
+          this);
+    }
+  }
+}
+
 void GOFrame::OnImportSettings(wxCommandEvent &event) {
   GOOrganController *organController = GetOrganController();
 
@@ -858,28 +949,12 @@ void GOFrame::OnImportSettings(wxCommandEvent &event) {
       wxFD_OPEN | wxFD_FILE_MUST_EXIST);
 
     if (dlg.ShowModal() == wxID_OK) {
-      GOProgressDialog pdlg;
       GOOrgan organ = organController->GetOrganInfo();
       GOMutexLocker m_locker(m_mutex, true);
-      if (!m_locker.IsLocked())
-        return;
-      m_doc->Import(&pdlg, organ, dlg.GetPath());
-    }
-  }
-}
 
-void GOFrame::OnImportCombinations(wxCommandEvent &event) {
-  if (GetOrganController()) {
-    wxFileDialog dlg(
-      this,
-      _("Import Combinations"),
-      m_config.ExportImportPath(),
-      wxEmptyString,
-      _("Settings files (*.cmb)|*.cmb"),
-      wxFD_OPEN | wxFD_FILE_MUST_EXIST);
-
-    if (dlg.ShowModal() == wxID_OK) {
-      m_doc->ImportCombination(dlg.GetPath());
+      if (m_locker.IsLocked()) {
+        LoadOrgan(organ, dlg.GetPath());
+      }
     }
   }
 }
@@ -970,9 +1045,8 @@ void GOFrame::OnReload(wxCommandEvent &event) {
 }
 
 void GOFrame::OnMenuClose(wxCommandEvent &event) {
-  if (!m_doc)
-    return;
-  CloseOrgan(false);
+  if (m_doc)
+    CloseOrgan(false);
 }
 
 bool GOFrame::CloseProgram(bool isForce) {
@@ -998,10 +1072,11 @@ void GOFrame::OnRevert(wxCommandEvent &event) {
       wxYES_NO | wxICON_EXCLAMATION,
       this)
     == wxYES) {
-    if (m_doc) {
-      GOProgressDialog dlg;
+    GOOrganController *organController = GetOrganController();
 
-      m_doc->Revert(&dlg);
+    if (organController) {
+      organController->DeleteSettings();
+      LoadOrgan(organController->GetOrganInfo());
     }
   }
 }
@@ -1061,7 +1136,7 @@ void GOFrame::OnSettings(wxCommandEvent &event) {
 
   bool isToContinue = true; // will GO continue running? Otherwise it will exit
   SettingsReasons *const pReasons = (SettingsReasons *)event.GetClientData();
-  GOSettingsDialog dialog(this, m_Sound, pReasons);
+  GOSettingsDialog dialog(this, m_config, m_Sound, m_Sound.GetMidi(), pReasons);
 
   if (dialog.ShowModal() == wxID_OK) {
     GOArchiveManager manager(m_config, m_config.OrganCachePath());
@@ -1182,13 +1257,7 @@ void GOFrame::OnSettingsTranspose(wxCommandEvent &event) {
 }
 
 void GOFrame::OnSettingsReleaseLength(wxCommandEvent &event) {
-  m_config.ReleaseLength(m_ReleaseLength->GetSelection() * 50);
-  m_Sound.GetEngine().SetReleaseLength(m_config.ReleaseLength());
-
-  GOOrganController *organController = GetOrganController();
-
-  if (organController)
-    organController->SetReleaseTail(m_config.ReleaseLength());
+  UpdateReleaseLength(m_ReleaseLength->GetSelection() * 50);
 }
 
 void GOFrame::OnHelpAbout(wxCommandEvent &event) { DoSplash(false); }
