@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2022 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -8,11 +8,13 @@
 #include "GOOrganController.h"
 
 #include <math.h>
+#include <wx/datetime.h>
 #include <wx/filename.h>
 #include <wx/log.h>
 #include <wx/msgdlg.h>
-#include <wx/stream.h>
+#include <wx/txtstrm.h>
 #include <wx/wfstream.h>
+#include <yaml-cpp/yaml.h>
 
 #include "archive/GOArchive.h"
 #include "archive/GOArchiveFile.h"
@@ -48,6 +50,8 @@
 #include "midi/GOMidiEvent.h"
 #include "midi/GOMidiPlayer.h"
 #include "midi/GOMidiRecorder.h"
+#include "model/GOCoupler.h"
+#include "model/GODivisionalCoupler.h"
 #include "model/GOEnclosure.h"
 #include "model/GOManual.h"
 #include "model/GORank.h"
@@ -58,13 +62,14 @@
 #include "sound/GOSoundEngine.h"
 #include "sound/GOSoundReleaseAlignTable.h"
 #include "temperaments/GOTemperament.h"
+#include "yaml/go-wx-yaml.h"
+
+#include "go_defs.h"
 
 #include "GOAudioRecorder.h"
 #include "GOBuffer.h"
 #include "GOCache.h"
 #include "GOCacheWriter.h"
-#include "GOCoupler.h"
-#include "GODivisionalCoupler.h"
 #include "GODocument.h"
 #include "GOEvent.h"
 #include "GOHash.h"
@@ -72,14 +77,18 @@
 #include "GOOrgan.h"
 #include "GOPath.h"
 
-GOOrganController::GOOrganController(GODocument *doc, GOConfig &settings)
+static const wxString WX_ORGAN = wxT("Organ");
+static const wxString WX_GRANDORGUE_VERSION = wxT("GrandOrgueVersion");
+
+GOOrganController::GOOrganController(
+  GOConfig &config, GOMidiDialogCreator *pMidiDialogCreator)
   : GOEventDistributor(this),
-    GOOrganModel(settings),
-    m_doc(doc),
+    GOOrganModel(config),
+    m_config(config),
     m_odf(),
     m_ArchiveID(),
     m_hash(),
-    m_FileStore(settings),
+    m_FileStore(config),
     m_CacheFilename(),
     m_SettingFilename(),
     m_ODFHash(),
@@ -92,11 +101,6 @@ GOOrganController::GOOrganController(GODocument *doc, GOConfig &settings)
     m_b_customized(false),
     m_CurrentPitch(999999.0), // for enforcing updating the label first time
     m_OrganModified(false),
-    m_DivisionalsStoreIntermanualCouplers(false),
-    m_DivisionalsStoreIntramanualCouplers(false),
-    m_DivisionalsStoreTremulants(false),
-    m_GeneralsStoreDivisionalCouplers(false),
-    m_CombinationsStoreNonDisplayedDrawstops(false),
     m_ChurchName(),
     m_ChurchAddress(),
     m_OrganBuilder(),
@@ -107,19 +111,18 @@ GOOrganController::GOOrganController(GODocument *doc, GOConfig &settings)
     m_panels(),
     m_panelcreators(),
     m_elementcreators(),
-    m_UsedSections(),
     m_soundengine(0),
     m_midi(0),
     m_MidiSamplesetMatch(),
     m_SampleSetId1(0),
     m_SampleSetId2(0),
     m_bitmaps(this),
-    m_config(settings),
-    m_GeneralTemplate(this),
+    m_GeneralTemplate(*this),
     m_PitchLabel(this),
     m_TemperamentLabel(this),
     m_MainWindowData(this, wxT("MainWindow")) {
-  GOOrganModel::SetModificationListener(this);
+  GOOrganModel::SetMidiDialogCreator(pMidiDialogCreator);
+  GOOrganModel::SetModelModificationListener(this);
   m_pool.SetMemoryLimit(m_config.MemoryLimit() * 1024 * 1024);
 }
 
@@ -130,7 +133,8 @@ GOOrganController::~GOOrganController() {
   m_manuals.clear();
   m_tremulants.clear();
   m_ranks.clear();
-  GOOrganModel::SetModificationListener(nullptr);
+  GOOrganModel::SetModelModificationListener(nullptr);
+  GOOrganModel::SetMidiDialogCreator(nullptr);
 }
 
 void GOOrganController::SetOrganModified(bool modified) {
@@ -138,12 +142,14 @@ void GOOrganController::SetOrganModified(bool modified) {
     m_OrganModified = modified;
     m_setter->UpdateModified(modified);
   }
+  GOModificationProxy::OnIsModifiedChanged(modified);
 }
 
 void GOOrganController::OnIsModifiedChanged(bool modified) {
   if (modified) {
     // Update the pitch label if it has been changed
-    const float newPitch = GetRootPipeConfigNode().GetPipeConfig().GetTuning();
+    const float newPitch
+      = GetRootPipeConfigNode().GetPipeConfig().GetManualTuning();
 
     if (newPitch != m_CurrentPitch) {
       m_PitchLabel.SetContent(wxString::Format(_("%f cent"), newPitch));
@@ -182,23 +188,21 @@ GOHashType GOOrganController::GenerateCacheHash() {
 }
 
 void GOOrganController::ReadOrganFile(GOConfigReader &cfg) {
-  wxString group = wxT("Organ");
-
   /* load church info */
   cfg.ReadString(
-    ODFSetting, group, wxT("HauptwerkOrganFileFormatVersion"), false);
-  m_ChurchName = cfg.ReadStringTrim(ODFSetting, group, wxT("ChurchName"));
-  m_ChurchAddress = cfg.ReadString(ODFSetting, group, wxT("ChurchAddress"));
+    ODFSetting, WX_ORGAN, wxT("HauptwerkOrganFileFormatVersion"), false);
+  m_ChurchName = cfg.ReadStringTrim(ODFSetting, WX_ORGAN, wxT("ChurchName"));
+  m_ChurchAddress = cfg.ReadString(ODFSetting, WX_ORGAN, wxT("ChurchAddress"));
   m_OrganBuilder
-    = cfg.ReadString(ODFSetting, group, wxT("OrganBuilder"), false);
+    = cfg.ReadString(ODFSetting, WX_ORGAN, wxT("OrganBuilder"), false);
   m_OrganBuildDate
-    = cfg.ReadString(ODFSetting, group, wxT("OrganBuildDate"), false);
+    = cfg.ReadString(ODFSetting, WX_ORGAN, wxT("OrganBuildDate"), false);
   m_OrganComments
-    = cfg.ReadString(ODFSetting, group, wxT("OrganComments"), false);
+    = cfg.ReadString(ODFSetting, WX_ORGAN, wxT("OrganComments"), false);
   m_RecordingDetails
-    = cfg.ReadString(ODFSetting, group, wxT("RecordingDetails"), false);
+    = cfg.ReadString(ODFSetting, WX_ORGAN, wxT("RecordingDetails"), false);
   wxString info_filename
-    = cfg.ReadFileName(ODFSetting, group, wxT("InfoFilename"), false);
+    = cfg.ReadFileName(ODFSetting, WX_ORGAN, wxT("InfoFilename"), false);
   wxFileName fn;
   if (info_filename.IsEmpty()) {
     /* Resolve organ file path */
@@ -229,38 +233,22 @@ void GOOrganController::ReadOrganFile(GOConfigReader &cfg) {
   }
 
   /* load basic organ information */
-  unsigned NumberOfPanels
-    = cfg.ReadInteger(ODFSetting, group, wxT("NumberOfPanels"), 0, 100, false);
-  m_DivisionalsStoreIntermanualCouplers = cfg.ReadBoolean(
-    ODFSetting, group, wxT("DivisionalsStoreIntermanualCouplers"));
-  m_DivisionalsStoreIntramanualCouplers = cfg.ReadBoolean(
-    ODFSetting, group, wxT("DivisionalsStoreIntramanualCouplers"));
-  m_DivisionalsStoreTremulants
-    = cfg.ReadBoolean(ODFSetting, group, wxT("DivisionalsStoreTremulants"));
-  m_GeneralsStoreDivisionalCouplers = cfg.ReadBoolean(
-    ODFSetting, group, wxT("GeneralsStoreDivisionalCouplers"));
-  m_CombinationsStoreNonDisplayedDrawstops = cfg.ReadBoolean(
-    ODFSetting,
-    group,
-    wxT("CombinationsStoreNonDisplayedDrawstops"),
-    false,
-    true);
+  unsigned NumberOfPanels = cfg.ReadInteger(
+    ODFSetting, WX_ORGAN, wxT("NumberOfPanels"), 0, 100, false);
+  cfg.ReadString(CMBSetting, WX_ORGAN, WX_GRANDORGUE_VERSION, false);
   m_volume = cfg.ReadInteger(
-    CMBSetting, group, wxT("Volume"), -120, 100, false, m_config.Volume());
+    CMBSetting, WX_ORGAN, wxT("Volume"), -120, 100, false, m_config.Volume());
   if (m_volume > 20)
     m_volume = 0;
-  m_Temperament = cfg.ReadString(CMBSetting, group, wxT("Temperament"), false);
-  m_releaseTail = (unsigned)cfg.ReadInteger(
-    CMBSetting,
-    group,
-    wxT("ReleaseTail"),
-    0,
-    3000,
-    false,
-    m_config.ReleaseLength());
+  m_Temperament
+    = cfg.ReadString(CMBSetting, WX_ORGAN, wxT("Temperament"), false);
+
+  // It must be created before GOOrganModel::Load because lots of objects
+  // reference to it
+  m_setter = new GOSetter(this);
+  m_elementcreators.push_back(m_setter);
 
   GOOrganModel::Load(cfg, this);
-  wxString buffer;
 
   for (unsigned i = 0; i < m_enclosures.size(); i++)
     m_enclosures[i]->SetElementID(
@@ -274,9 +262,7 @@ void GOOrganController::ReadOrganFile(GOConfigReader &cfg) {
     m_tremulants[i]->SetElementID(
       GetRecorderElementID(wxString::Format(wxT("T%d"), i)));
 
-  m_setter = new GOSetter(this);
-  m_elementcreators.push_back(m_setter);
-  m_DivisionalSetter = new GODivisionalSetter(this);
+  m_DivisionalSetter = new GODivisionalSetter(this, m_setter->GetState());
   m_elementcreators.push_back(m_DivisionalSetter);
   m_AudioRecorder = new GOAudioRecorder(this);
   m_MidiRecorder = new GOMidiRecorder(this);
@@ -307,6 +293,8 @@ void GOOrganController::ReadOrganFile(GOConfigReader &cfg) {
   m_panels.push_back(new GOGUIPanel(this));
   m_panels[0]->Load(cfg, wxT(""));
 
+  wxString buffer;
+
   for (unsigned i = 0; i < NumberOfPanels; i++) {
     buffer.Printf(wxT("Panel%03d"), i + 1);
     m_panels.push_back(new GOGUIPanel(this));
@@ -325,6 +313,7 @@ void GOOrganController::ReadOrganFile(GOConfigReader &cfg) {
 
   GetRootPipeConfigNode().SetName(GetChurchName());
   ReadCombinations(cfg);
+  m_setter->OnCombinationsLoaded(GetCombinationsDir(), wxEmptyString);
 
   GOHash hash;
   hash.Update(m_ChurchName.utf8_str(), strlen(m_ChurchName.utf8_str()));
@@ -364,7 +353,7 @@ wxString GOOrganController::Load(
     if (!m_FileStore.LoadArchives(
           m_config, m_config.OrganCachePath(), organ.GetArchiveID(), errMsg))
       return errMsg;
-
+    m_ArchivePath = organ.GetArchivePath();
     m_odf = organ.GetODFPath();
     odf_name.Assign(m_odf);
   } else {
@@ -435,14 +424,14 @@ wxString GOOrganController::Load(
     }
 
     if (
-      odf_ini_file.getEntry(wxT("Organ"), wxT("ChurchName")).Trim()
-      != extra_odf_config.getEntry(wxT("Organ"), wxT("ChurchName")).Trim())
+      odf_ini_file.getEntry(WX_ORGAN, wxT("ChurchName")).Trim()
+      != extra_odf_config.getEntry(WX_ORGAN, wxT("ChurchName")).Trim())
       wxLogWarning(
         _("This .cmb file was originally created for:\n%s"),
-        extra_odf_config.getEntry(wxT("Organ"), wxT("ChurchName")).c_str());
+        extra_odf_config.getEntry(WX_ORGAN, wxT("ChurchName")).c_str());
 
     ini.ReadData(extra_odf_config, CMBSetting, false);
-    wxString hash = extra_odf_config.getEntry(wxT("Organ"), wxT("ODFHash"));
+    wxString hash = extra_odf_config.getEntry(WX_ORGAN, wxT("ODFHash"));
     if (hash != wxEmptyString)
       if (hash != m_ODFHash) {
         if (
@@ -473,13 +462,13 @@ wxString GOOrganController::Load(
   }
 
   try {
-    GOConfigReader cfg(ini, m_config.ODFCheck());
+    GOConfigReader cfg(ini, m_config.ODFCheck(), m_config.ODFHw1Check());
     /* skip informational items */
-    cfg.ReadString(CMBSetting, wxT("Organ"), wxT("ChurchName"), false);
-    cfg.ReadString(CMBSetting, wxT("Organ"), wxT("ChurchAddress"), false);
-    cfg.ReadString(CMBSetting, wxT("Organ"), wxT("ODFPath"), false);
-    cfg.ReadString(CMBSetting, wxT("Organ"), wxT("ODFHash"), false);
-    cfg.ReadString(CMBSetting, wxT("Organ"), wxT("ArchiveID"), false);
+    cfg.ReadString(CMBSetting, WX_ORGAN, wxT("ChurchName"), false);
+    cfg.ReadString(CMBSetting, WX_ORGAN, wxT("ChurchAddress"), false);
+    cfg.ReadString(CMBSetting, WX_ORGAN, wxT("ODFPath"), false);
+    cfg.ReadString(CMBSetting, WX_ORGAN, wxT("ODFHash"), false);
+    cfg.ReadString(CMBSetting, WX_ORGAN, wxT("ArchiveID"), false);
     ReadOrganFile(cfg);
   } catch (wxString error_) {
     return error_;
@@ -545,14 +534,9 @@ wxString GOOrganController::Load(
           cache_ok = false;
       }
 
-      if (!cache_ok && !m_config.ManageCache()) {
-        GOMessageBox(
-          _("The cache for this organ is outdated. Please update or "
-            "delete it."),
-          _("Warning"),
-          wxOK | wxICON_WARNING,
-          NULL);
-      }
+      if (!cache_ok && !m_config.ManageCache())
+        wxLogWarning(_(
+          "The cache for this organ is outdated. Please update or delete it."));
 
       reader.Close();
     }
@@ -628,45 +612,137 @@ wxString GOOrganController::Load(
   return errMsg;
 }
 
+// const wxString &WX_CMB = wxT(".cmb");
+const wxString &WX_YAML = wxT("yaml");
+const char *const INFO = "info";
+const char *const CONTENT_TYPE = "content-type";
+const wxString WX_GRANDORGUE_COMBINATIONS = "GrandOrgue Combinations";
+const char *const ORGAN_NAME = "organ-name";
+const char *const GRANDORGUE_VERSION = "grandorgue-version";
+const char *const SAVED_TIME = "saved_time";
+
+wxString GOOrganController::ExportCombination(const wxString &fileName) {
+  wxString errMsg;
+  wxFileOutputStream fOS(fileName);
+
+  if (fOS.IsOk()) {
+    YAML::Node globalNode(YAML::NodeType::Map);
+    YAML::Node infoNode = globalNode[INFO];
+
+    infoNode[CONTENT_TYPE] = WX_GRANDORGUE_COMBINATIONS;
+    infoNode[ORGAN_NAME] = m_ChurchName;
+    infoNode[GRANDORGUE_VERSION] = APP_VERSION;
+    infoNode[SAVED_TIME] = wxDateTime::Now().Format();
+
+    globalNode << *m_setter;
+    globalNode << *m_DivisionalSetter;
+    YAML::Emitter outYaml;
+
+    outYaml << YAML::BeginDoc << globalNode;
+
+    if (fOS.WriteAll(outYaml.c_str(), outYaml.size()))
+      m_setter->OnCombinationsSaved(fileName);
+    else
+      errMsg.Printf(
+        wxT("Unable to write all the data to the file '%s'"), fileName);
+    fOS.Close();
+  } else
+    errMsg.Printf(wxT("Unable to open the file '%s' for writing"), fileName);
+  return errMsg;
+}
+
+/**
+ * Check the churchName of the imported combination file. If it differs from the
+ * current organ m_ChurchName then ask for the user
+ * @param churchName the organ the combination file was saved of
+ * @return true if the churchNames are the same or the user agree with importing
+ *   the combination file
+ */
+bool GOOrganController::IsToImportCombinationsFor(
+  const wxString &fileName, const wxString &churchName) const {
+  bool isToImport = true;
+
+  if (churchName != m_ChurchName) {
+    wxLogWarning(
+      _("This combination file '%s' was originally made for another organ "
+        "'%s'"),
+      fileName,
+      churchName);
+    isToImport = wxMessageBox(
+                   wxString::Format(
+                     _("This combination file '%s' was originally made for "
+                       "another organ '%s'. Importing it can cause various "
+                       "problems. Should it really be imported?"),
+                     fileName,
+                     churchName),
+                   _("Import Combinations"),
+                   wxYES_NO,
+                   NULL)
+      == wxYES;
+  }
+  return isToImport;
+}
+
 void GOOrganController::LoadCombination(const wxString &file) {
+  wxString errMsg;
+  const wxFileName fileName(file);
+
   try {
-    GOConfigFileReader odf_ini_file;
+    const wxString fileExt = fileName.GetExt();
 
-    if (!odf_ini_file.Read(file))
-      throw wxString::Format(_("Unable to read '%s'"), file.c_str());
+    if (fileExt == WX_YAML) {
+      YAML::Node cmbNode = YAML::LoadFile(file.c_str().AsChar());
+      YAML::Node cmbInfoNode = cmbNode[INFO];
+      const wxString contentType = cmbInfoNode[CONTENT_TYPE].as<wxString>();
 
-    GOConfigReaderDB ini;
-    ini.ReadData(odf_ini_file, CMBSetting, false);
-    GOConfigReader cfg(ini);
+      if (contentType != WX_GRANDORGUE_COMBINATIONS)
+        throw wxString::Format(
+          _("The file '%s' is not a GrandOrgue Combination file"), file);
+      if (IsToImportCombinationsFor(
+            file, cmbInfoNode[ORGAN_NAME].as<wxString>())) {
+        cmbNode >> *m_setter;
+        cmbNode >> *m_DivisionalSetter;
+        m_setter->OnCombinationsLoaded(fileName.GetPath(), file);
+      }
+    } else {
+      GOConfigFileReader odf_ini_file;
 
-    wxString church_name
-      = cfg.ReadString(CMBSetting, wxT("Organ"), wxT("ChurchName"));
-    if (church_name != m_ChurchName)
-      if (
-        wxMessageBox(
-          _("This combination file was originally made for "
-            "another organ. Importing it can cause various "
-            "problems. Should it really be imported?"),
-          _("Import"),
-          wxYES_NO,
-          NULL)
-        == wxNO)
+      if (!odf_ini_file.Read(file))
+        throw wxString::Format(_("Unable to read '%s'"), file.c_str());
+
+      GOConfigReaderDB ini;
+      ini.ReadData(odf_ini_file, CMBSetting, false);
+      GOConfigReader cfg(ini);
+
+      wxString church_name
+        = cfg.ReadString(CMBSetting, WX_ORGAN, wxT("ChurchName"));
+      if (!IsToImportCombinationsFor(file, church_name))
         return;
 
-    wxString hash = odf_ini_file.getEntry(wxT("Organ"), wxT("ODFHash"));
-    if (hash != wxEmptyString)
-      if (hash != m_ODFHash) {
-        wxLogWarning(
-          _("The combination file does not exactly match the current ODF."));
-      }
-    /* skip informational items */
-    cfg.ReadString(CMBSetting, wxT("Organ"), wxT("ChurchAddress"), false);
-    cfg.ReadString(CMBSetting, wxT("Organ"), wxT("ODFPath"), false);
+      wxString hash = odf_ini_file.getEntry(WX_ORGAN, wxT("ODFHash"));
+      if (hash != wxEmptyString)
+        if (hash != m_ODFHash) {
+          wxLogWarning(
+            _("The combination file does not exactly match the current ODF."));
+        }
+      /* skip informational items */
+      cfg.ReadString(CMBSetting, WX_ORGAN, wxT("ChurchAddress"), false);
+      cfg.ReadString(CMBSetting, WX_ORGAN, wxT("ODFPath"), false);
 
-    ReadCombinations(cfg);
-  } catch (wxString error) {
-    wxLogError(wxT("%s\n"), error.c_str());
-    GOMessageBox(error, _("Load error"), wxOK | wxICON_ERROR, NULL);
+      ReadCombinations(cfg);
+      m_setter->OnCombinationsLoaded(GetCombinationsDir(), wxEmptyString);
+    }
+    SetOrganModified();
+  } catch (const wxString &error) {
+    errMsg = error;
+  } catch (const std::exception &e) {
+    errMsg = e.what();
+  } catch (...) { // We must not allow unhandled exceptions here
+    errMsg.Printf("Unknown exception");
+  }
+  if (!errMsg.IsEmpty()) {
+    wxLogError(errMsg);
+    GOMessageBox(errMsg, _("Load error"), wxOK | wxICON_ERROR, NULL);
   }
 }
 
@@ -730,38 +806,33 @@ bool GOOrganController::Save() {
 }
 
 bool GOOrganController::Export(const wxString &cmb) {
-  wxString fn = cmb;
-  wxString tmp_name = fn + wxT(".new");
-  wxString buffer;
-  bool prefix = false;
-
   GOConfigFileWriter cfg_file;
+  GOConfigWriter cfg(cfg_file, false);
+
   m_b_customized = true;
-
-  GOConfigWriter cfg(cfg_file, prefix);
-  cfg.WriteString(wxT("Organ"), wxT("ODFHash"), m_ODFHash);
-  cfg.WriteString(wxT("Organ"), wxT("ChurchName"), m_ChurchName);
-  cfg.WriteString(wxT("Organ"), wxT("ChurchAddress"), m_ChurchAddress);
-  cfg.WriteString(wxT("Organ"), wxT("ODFPath"), GetODFFilename());
+  cfg.WriteString(WX_ORGAN, wxT("ODFHash"), m_ODFHash);
+  cfg.WriteString(WX_ORGAN, wxT("ChurchName"), m_ChurchName);
+  cfg.WriteString(WX_ORGAN, wxT("ChurchAddress"), m_ChurchAddress);
+  cfg.WriteString(WX_ORGAN, wxT("ODFPath"), GetODFFilename());
   if (m_ArchiveID != wxEmptyString)
-    cfg.WriteString(wxT("Organ"), wxT("ArchiveID"), m_ArchiveID);
-
-  cfg.WriteInteger(wxT("Organ"), wxT("Volume"), m_volume);
-
-  cfg.WriteString(wxT("Organ"), wxT("Temperament"), m_Temperament);
-  cfg.WriteInteger(wxT("Organ"), wxT("ReleaseTail"), (int)m_releaseTail);
+    cfg.WriteString(WX_ORGAN, wxT("ArchiveID"), m_ArchiveID);
+  cfg.WriteString(WX_ORGAN, WX_GRANDORGUE_VERSION, wxT(APP_VERSION));
+  cfg.WriteInteger(WX_ORGAN, wxT("Volume"), m_volume);
+  cfg.WriteString(WX_ORGAN, wxT("Temperament"), m_Temperament);
 
   GOEventDistributor::Save(cfg);
 
+  wxString tmp_name = cmb + wxT(".new");
+
   if (::wxFileExists(tmp_name) && !::wxRemoveFile(tmp_name)) {
-    wxLogError(_("Could not write to '%s'"), tmp_name.c_str());
+    wxLogError(_("Could not write to '%s'"), tmp_name);
     return false;
   }
   if (!cfg_file.Save(tmp_name)) {
-    wxLogError(_("Could not write to '%s'"), tmp_name.c_str());
+    wxLogError(_("Could not write to '%s'"), tmp_name);
     return false;
   }
-  if (!GORenameFile(tmp_name, fn))
+  if (!GORenameFile(tmp_name, cmb))
     return false;
   return true;
 }
@@ -796,37 +867,12 @@ GOButtonControl *GOOrganController::GetButtonControl(
   return NULL;
 }
 
-GODocument *GOOrganController::GetDocument() { return m_doc; }
+// GODocument *GOOrganController::GetDocument() { return m_doc; }
 
 void GOOrganController::SetVolume(int volume) { m_volume = volume; }
 
 int GOOrganController::GetVolume() { return m_volume; }
 
-void GOOrganController::SetReleaseTail(unsigned releaseTail) {
-  m_releaseTail = releaseTail;
-  SetOrganModified();
-}
-
-bool GOOrganController::DivisionalsStoreIntermanualCouplers() {
-  return m_DivisionalsStoreIntermanualCouplers;
-}
-
-bool GOOrganController::DivisionalsStoreIntramanualCouplers() {
-  return m_DivisionalsStoreIntramanualCouplers;
-}
-
-bool GOOrganController::DivisionalsStoreTremulants() {
-  return m_DivisionalsStoreTremulants;
-}
-
-bool GOOrganController::CombinationsStoreNonDisplayedDrawstops() {
-  return m_CombinationsStoreNonDisplayedDrawstops;
-}
-
-bool GOOrganController::GeneralsStoreDivisionalCouplers() {
-  return m_GeneralsStoreDivisionalCouplers;
-}
-
 GOSetter *GOOrganController::GetSetter() { return m_setter; }
 
 GOGUIPanel *GOOrganController::GetPanel(unsigned index) {
@@ -894,54 +940,17 @@ const wxString GOOrganController::GetSettingFilename() {
 
 const wxString GOOrganController::GetCacheFilename() { return m_CacheFilename; }
 
+wxString GOOrganController::GetCombinationsDir() const {
+  return wxFileName(m_config.OrganCombinationsPath(), m_ChurchName)
+    .GetFullPath();
+}
+
 GOMemoryPool &GOOrganController::GetMemoryPool() { return m_pool; }
 
 GOConfig &GOOrganController::GetSettings() { return m_config; }
 
 GOBitmapCache &GOOrganController::GetBitmapCache() { return m_bitmaps; }
 
-GOSoundSampler *GOOrganController::StartSample(
-  const GOSoundProvider *pipe,
-  int sampler_group_id,
-  unsigned audio_group,
-  unsigned velocity,
-  unsigned delay,
-  uint64_t last_stop) {
-  if (!m_soundengine)
-    return NULL;
-  return m_soundengine->StartSample(
-    pipe, sampler_group_id, audio_group, velocity, delay, last_stop);
-}
-
-uint64_t GOOrganController::StopSample(
-  const GOSoundProvider *pipe, GOSoundSampler *handle) {
-  if (m_soundengine)
-    return m_soundengine->StopSample(pipe, handle);
-  return 0;
-}
-
-void GOOrganController::SwitchSample(
-  const GOSoundProvider *pipe, GOSoundSampler *handle) {
-  if (m_soundengine)
-    m_soundengine->SwitchSample(pipe, handle);
-}
-
-void GOOrganController::UpdateVelocity(
-  const GOSoundProvider *pipe, GOSoundSampler *handle, unsigned velocity) {
-  if (m_soundengine)
-    m_soundengine->UpdateVelocity(pipe, handle, velocity);
-}
-
-void GOOrganController::SendMidiMessage(GOMidiEvent &e) {
-  if (m_midi)
-    m_midi->Send(e);
-}
-
-void GOOrganController::SendMidiRecorderMessage(GOMidiEvent &e) {
-  if (m_MidiRecorder)
-    m_MidiRecorder->SendMidiRecorderMessage(e);
-}
-
 GOMidi *GOOrganController::GetMidi() { return m_midi; }
 
 void GOOrganController::LoadMIDIFile(wxString const &filename) {
@@ -958,6 +967,7 @@ void GOOrganController::Abort() {
   m_MidiRecorder->StopRecording();
   m_AudioRecorder->StopRecording();
   m_AudioRecorder->SetAudioRecorder(NULL);
+  GOOrganModel::SetMidi(nullptr, nullptr);
   m_midi = NULL;
 }
 
@@ -981,7 +991,8 @@ void GOOrganController::PreparePlayback(
   PreconfigRecorder();
 
   m_MidiSamplesetMatch.clear();
-  GOEventDistributor::PreparePlayback();
+  GOOrganModel::SetMidi(midi, m_MidiRecorder);
+  GOEventDistributor::PreparePlayback(engine);
 
   m_setter->UpdateModified(m_OrganModified);
 
@@ -1051,7 +1062,7 @@ void GOOrganController::Reset() {
     GetDivisionalCoupler(j)->Reset();
   for (unsigned k = 0; k < GetGeneralCount(); k++)
     GetGeneral(k)->Display(false);
-  m_setter->ResetDisplay();
+  m_setter->ResetCmbButtons();
 }
 
 void GOOrganController::SetTemperament(const GOTemperament &temperament) {
@@ -1091,9 +1102,3 @@ GOLabelControl *GOOrganController::GetTemperamentLabel() {
 GOMainWindowData *GOOrganController::GetMainWindowData() {
   return &m_MainWindowData;
 }
-
-void GOOrganController::MarkSectionInUse(wxString name) {
-  if (m_UsedSections[name])
-    throw wxString::Format(_("Section %s already in use"), name.c_str());
-  m_UsedSections[name] = true;
-}
