@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2022 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -8,14 +8,21 @@
 #include "GOSetter.h"
 
 #include <wx/app.h>
+#include <wx/dir.h>
+#include <wx/filename.h>
 #include <wx/intl.h>
 #include <wx/window.h>
+#include <yaml-cpp/yaml.h>
 
-#include "combinations/model/GOGeneralCombination.h"
 #include "config/GOConfig.h"
 #include "config/GOConfigReader.h"
 #include "config/GOConfigWriter.h"
 #include "control/GOCallbackButtonControl.h"
+#include "control/GODivisionalButtonControl.h"
+#include "control/GOGeneralButtonControl.h"
+#include "model/GODivisionalCoupler.h"
+#include "model/GOManual.h"
+#include "yaml/go-wx-yaml.h"
 
 #include "GOEvent.h"
 #include "GOOrganController.h"
@@ -56,6 +63,13 @@ enum {
   ID_SETTER_DELETE,
   ID_SETTER_INSERT,
 
+  ID_SETTER_REFRESH_FILES,
+  ID_SETTER_PREV_FILE,
+  ID_SETTER_NEXT_FILE,
+  ID_SETTER_LOAD_FILE,
+  ID_SETTER_SAVE_FILE,
+  ID_SETTER_SAVE_SETTINGS,
+
   ID_SETTER_GENERAL00,
   ID_SETTER_GENERAL01,
   ID_SETTER_GENERAL02,
@@ -129,10 +143,25 @@ enum {
   ID_SETTER_TRANSPOSE_DOWN,
   ID_SETTER_TRANSPOSE_UP,
 
-  ID_SETTER_SAVE,
   ID_SETTER_ON,
 };
 
+const wxString GOSetter::KEY_REFRESH_FILES = wxT("RefreshFiles");
+const wxString GOSetter::KEY_PREV_FILE = wxT("PrevFile");
+const wxString GOSetter::KEY_CURR_FILE_NAME = wxT("CurrFileName");
+const wxString GOSetter::KEY_NEXT_FILE = wxT("NextFile");
+const wxString GOSetter::KEY_LOAD_FILE = wxT("LoadFile");
+const wxString GOSetter::KEY_SAVE_FILE = wxT("SaveFile");
+const wxString GOSetter::KEY_SAVE_SETTINGS = wxT("Save");
+
+const wxString GOSetter::GROUP_REFRESH_FILES = wxT("SetterRefreshFiles");
+const wxString GOSetter::GROUP_PREV_FILE = wxT("SetterPrevFile");
+const wxString GOSetter::GROUP_CURR_FILE_NAME = wxT("SetterCurrFileName");
+const wxString GOSetter::GROUP_NEXT_FILE = wxT("SetterNextFile");
+const wxString GOSetter::GROUP_LOAD_FILE = wxT("SetterLoadFile");
+const wxString GOSetter::GROUP_SAVE_FILE = wxT("SetterSaveFile");
+const wxString GOSetter::GROUP_SAVE_SETTINGS = wxT("SetterSave");
+
 const struct GOElementCreator::ButtonDefinitionEntry GOSetter::m_element_types[]
   = {
     {wxT("Prev"), ID_SETTER_PREV, true, true, false},
@@ -163,56 +192,61 @@ const struct GOElementCreator::ButtonDefinitionEntry GOSetter::m_element_types[]
     {wxT("Full"), ID_SETTER_FULL, true, false, false},
     {wxT("Insert"), ID_SETTER_INSERT, true, true, false},
     {wxT("Delete"), ID_SETTER_DELETE, true, true, false},
-    {wxT("General01"), ID_SETTER_GENERAL00, true, true, false},
-    {wxT("General02"), ID_SETTER_GENERAL01, true, true, false},
-    {wxT("General03"), ID_SETTER_GENERAL02, true, true, false},
-    {wxT("General04"), ID_SETTER_GENERAL03, true, true, false},
-    {wxT("General05"), ID_SETTER_GENERAL04, true, true, false},
-    {wxT("General06"), ID_SETTER_GENERAL05, true, true, false},
-    {wxT("General07"), ID_SETTER_GENERAL06, true, true, false},
-    {wxT("General08"), ID_SETTER_GENERAL07, true, true, false},
-    {wxT("General09"), ID_SETTER_GENERAL08, true, true, false},
-    {wxT("General10"), ID_SETTER_GENERAL09, true, true, false},
-    {wxT("General11"), ID_SETTER_GENERAL10, true, true, false},
-    {wxT("General12"), ID_SETTER_GENERAL11, true, true, false},
-    {wxT("General13"), ID_SETTER_GENERAL12, true, true, false},
-    {wxT("General14"), ID_SETTER_GENERAL13, true, true, false},
-    {wxT("General15"), ID_SETTER_GENERAL14, true, true, false},
-    {wxT("General16"), ID_SETTER_GENERAL15, true, true, false},
-    {wxT("General17"), ID_SETTER_GENERAL16, true, true, false},
-    {wxT("General18"), ID_SETTER_GENERAL17, true, true, false},
-    {wxT("General19"), ID_SETTER_GENERAL18, true, true, false},
-    {wxT("General20"), ID_SETTER_GENERAL19, true, true, false},
-    {wxT("General21"), ID_SETTER_GENERAL20, true, true, false},
-    {wxT("General22"), ID_SETTER_GENERAL21, true, true, false},
-    {wxT("General23"), ID_SETTER_GENERAL22, true, true, false},
-    {wxT("General24"), ID_SETTER_GENERAL23, true, true, false},
-    {wxT("General25"), ID_SETTER_GENERAL24, true, true, false},
-    {wxT("General26"), ID_SETTER_GENERAL25, true, true, false},
-    {wxT("General27"), ID_SETTER_GENERAL26, true, true, false},
-    {wxT("General28"), ID_SETTER_GENERAL27, true, true, false},
-    {wxT("General29"), ID_SETTER_GENERAL28, true, true, false},
-    {wxT("General30"), ID_SETTER_GENERAL29, true, true, false},
-    {wxT("General31"), ID_SETTER_GENERAL30, true, true, false},
-    {wxT("General32"), ID_SETTER_GENERAL31, true, true, false},
-    {wxT("General33"), ID_SETTER_GENERAL32, true, true, false},
-    {wxT("General34"), ID_SETTER_GENERAL33, true, true, false},
-    {wxT("General35"), ID_SETTER_GENERAL34, true, true, false},
-    {wxT("General36"), ID_SETTER_GENERAL35, true, true, false},
-    {wxT("General37"), ID_SETTER_GENERAL36, true, true, false},
-    {wxT("General38"), ID_SETTER_GENERAL37, true, true, false},
-    {wxT("General39"), ID_SETTER_GENERAL38, true, true, false},
-    {wxT("General40"), ID_SETTER_GENERAL39, true, true, false},
-    {wxT("General41"), ID_SETTER_GENERAL40, true, true, false},
-    {wxT("General42"), ID_SETTER_GENERAL41, true, true, false},
-    {wxT("General43"), ID_SETTER_GENERAL42, true, true, false},
-    {wxT("General44"), ID_SETTER_GENERAL43, true, true, false},
-    {wxT("General45"), ID_SETTER_GENERAL44, true, true, false},
-    {wxT("General46"), ID_SETTER_GENERAL45, true, true, false},
-    {wxT("General47"), ID_SETTER_GENERAL46, true, true, false},
-    {wxT("General48"), ID_SETTER_GENERAL47, true, true, false},
-    {wxT("General49"), ID_SETTER_GENERAL48, true, true, false},
-    {wxT("General50"), ID_SETTER_GENERAL49, true, true, false},
+    {KEY_REFRESH_FILES, ID_SETTER_REFRESH_FILES, true, true, false},
+    {KEY_PREV_FILE, ID_SETTER_PREV_FILE, true, true, false},
+    {KEY_NEXT_FILE, ID_SETTER_NEXT_FILE, true, true, false},
+    {KEY_LOAD_FILE, ID_SETTER_LOAD_FILE, true, true, false},
+    {KEY_SAVE_FILE, ID_SETTER_SAVE_FILE, true, true, false},
+    {wxT("General01"), ID_SETTER_GENERAL00, true, true, true},
+    {wxT("General02"), ID_SETTER_GENERAL01, true, true, true},
+    {wxT("General03"), ID_SETTER_GENERAL02, true, true, true},
+    {wxT("General04"), ID_SETTER_GENERAL03, true, true, true},
+    {wxT("General05"), ID_SETTER_GENERAL04, true, true, true},
+    {wxT("General06"), ID_SETTER_GENERAL05, true, true, true},
+    {wxT("General07"), ID_SETTER_GENERAL06, true, true, true},
+    {wxT("General08"), ID_SETTER_GENERAL07, true, true, true},
+    {wxT("General09"), ID_SETTER_GENERAL08, true, true, true},
+    {wxT("General10"), ID_SETTER_GENERAL09, true, true, true},
+    {wxT("General11"), ID_SETTER_GENERAL10, true, true, true},
+    {wxT("General12"), ID_SETTER_GENERAL11, true, true, true},
+    {wxT("General13"), ID_SETTER_GENERAL12, true, true, true},
+    {wxT("General14"), ID_SETTER_GENERAL13, true, true, true},
+    {wxT("General15"), ID_SETTER_GENERAL14, true, true, true},
+    {wxT("General16"), ID_SETTER_GENERAL15, true, true, true},
+    {wxT("General17"), ID_SETTER_GENERAL16, true, true, true},
+    {wxT("General18"), ID_SETTER_GENERAL17, true, true, true},
+    {wxT("General19"), ID_SETTER_GENERAL18, true, true, true},
+    {wxT("General20"), ID_SETTER_GENERAL19, true, true, true},
+    {wxT("General21"), ID_SETTER_GENERAL20, true, true, true},
+    {wxT("General22"), ID_SETTER_GENERAL21, true, true, true},
+    {wxT("General23"), ID_SETTER_GENERAL22, true, true, true},
+    {wxT("General24"), ID_SETTER_GENERAL23, true, true, true},
+    {wxT("General25"), ID_SETTER_GENERAL24, true, true, true},
+    {wxT("General26"), ID_SETTER_GENERAL25, true, true, true},
+    {wxT("General27"), ID_SETTER_GENERAL26, true, true, true},
+    {wxT("General28"), ID_SETTER_GENERAL27, true, true, true},
+    {wxT("General29"), ID_SETTER_GENERAL28, true, true, true},
+    {wxT("General30"), ID_SETTER_GENERAL29, true, true, true},
+    {wxT("General31"), ID_SETTER_GENERAL30, true, true, true},
+    {wxT("General32"), ID_SETTER_GENERAL31, true, true, true},
+    {wxT("General33"), ID_SETTER_GENERAL32, true, true, true},
+    {wxT("General34"), ID_SETTER_GENERAL33, true, true, true},
+    {wxT("General35"), ID_SETTER_GENERAL34, true, true, true},
+    {wxT("General36"), ID_SETTER_GENERAL35, true, true, true},
+    {wxT("General37"), ID_SETTER_GENERAL36, true, true, true},
+    {wxT("General38"), ID_SETTER_GENERAL37, true, true, true},
+    {wxT("General39"), ID_SETTER_GENERAL38, true, true, true},
+    {wxT("General40"), ID_SETTER_GENERAL39, true, true, true},
+    {wxT("General41"), ID_SETTER_GENERAL40, true, true, true},
+    {wxT("General42"), ID_SETTER_GENERAL41, true, true, true},
+    {wxT("General43"), ID_SETTER_GENERAL42, true, true, true},
+    {wxT("General44"), ID_SETTER_GENERAL43, true, true, true},
+    {wxT("General45"), ID_SETTER_GENERAL44, true, true, true},
+    {wxT("General46"), ID_SETTER_GENERAL45, true, true, true},
+    {wxT("General47"), ID_SETTER_GENERAL46, true, true, true},
+    {wxT("General48"), ID_SETTER_GENERAL47, true, true, true},
+    {wxT("General49"), ID_SETTER_GENERAL48, true, true, true},
+    {wxT("General50"), ID_SETTER_GENERAL49, true, true, true},
 
     {wxT("GeneralPrev"), ID_SETTER_GENERAL_PREV, true, true, false},
     {wxT("GeneralNext"), ID_SETTER_GENERAL_NEXT, true, true, false},
@@ -228,7 +262,7 @@ const struct GOElementCreator::ButtonDefinitionEntry GOSetter::m_element_types[]
     {wxT("TransposeDown"), ID_SETTER_TRANSPOSE_DOWN, true, true, false},
     {wxT("TransposeUp"), ID_SETTER_TRANSPOSE_UP, true, true, false},
 
-    {wxT("Save"), ID_SETTER_SAVE, true, true, false},
+    {KEY_SAVE_SETTINGS, ID_SETTER_SAVE_SETTINGS, true, true, false},
     {wxT("OnState"), ID_SETTER_ON, false, true, false},
 
     {wxT("CrescendoA"), ID_SETTER_CRESCENDO_A, true, true, false},
@@ -238,7 +272,11 @@ const struct GOElementCreator::ButtonDefinitionEntry GOSetter::m_element_types[]
     {wxT("CrescendoPrev"), ID_SETTER_CRESCENDO_PREV, true, true, false},
     {wxT("CrescendoCurrent"), ID_SETTER_CRESCENDO_CURRENT, true, true, false},
     {wxT("CrescendoNext"), ID_SETTER_CRESCENDO_NEXT, true, true, false},
-    {wxT("CrescendoOverride"), ID_SETTER_CRESCENDO_OVERRIDE, true, true, false},
+    {wxT("CrescendoOverride"),
+     ID_SETTER_CRESCENDO_OVERRIDE,
+     true,
+     false,
+     false},
     {wxT(""), -1, false, false, false},
 };
 
@@ -256,14 +294,14 @@ GOSetter::GOSetter(GOOrganController *organController)
     m_framegeneral(0),
     m_general(0),
     m_crescendo(0),
+    m_CurrFileDisplay(organController),
     m_PosDisplay(organController),
     m_BankDisplay(organController),
     m_CrescendoDisplay(organController),
     m_TransposeDisplay(organController),
     m_NameDisplay(organController),
-    m_swell(organController),
-    m_SetterType(SETTER_REGULAR) {
-  CreateButtons(m_OrganController);
+    m_swell(*organController) {
+  CreateButtons(*m_OrganController);
 
   m_buttons[ID_SETTER_PREV]->SetPreconfigIndex(0);
   m_buttons[ID_SETTER_NEXT]->SetPreconfigIndex(1);
@@ -295,16 +333,17 @@ GOSetter::GOSetter(GOOrganController *organController)
   m_buttons[ID_SETTER_NEXT]->SetShortcutKey(39);
   m_buttons[ID_SETTER_CURRENT]->SetShortcutKey(40);
 
-  SetSetterType(m_SetterType);
+  SetSetterType(GOSetterState::SETTER_REGULAR);
   SetCrescendoType(m_crescendobank);
 
-  m_OrganController->RegisterPlaybackStateHandler(this);
+  m_OrganController->RegisterSoundStateHandler(this);
+  m_OrganController->RegisterCombinationButtonSet(this);
   m_OrganController->RegisterControlChangedHandler(this);
 }
 
 GOSetter::~GOSetter() {}
 
-static const wxString OVERRIDE_MODE = wxT("OverrideMode");
+static const wxString WX_OVERRIDE_MODE = wxT("OverrideMode");
 
 void GOSetter::Load(GOConfigReader &cfg) {
   m_OrganController->RegisterSaveableObject(this);
@@ -314,7 +353,7 @@ void GOSetter::Load(GOConfigReader &cfg) {
   m_framegeneral.resize(0);
   for (unsigned i = 0; i < FRAME_GENERALS; i++) {
     m_framegeneral.push_back(new GOGeneralCombination(
-      m_OrganController->GetGeneralTemplate(), m_OrganController, true));
+      *m_OrganController, m_OrganController->GetGeneralTemplate(), true));
     buffer.Printf(wxT("FrameGeneral%03d"), i + 1);
     m_framegeneral[i]->Load(cfg, buffer);
   }
@@ -322,7 +361,7 @@ void GOSetter::Load(GOConfigReader &cfg) {
   m_general.resize(0);
   for (unsigned i = 0; i < GENERALS * GENERAL_BANKS; i++) {
     m_general.push_back(new GOGeneralCombination(
-      m_OrganController->GetGeneralTemplate(), m_OrganController, true));
+      *m_OrganController, m_OrganController->GetGeneralTemplate(), true));
     buffer.Printf(wxT("SetterGeneral%03d"), i + 1);
     m_general[i]->Load(cfg, buffer);
   }
@@ -332,14 +371,14 @@ void GOSetter::Load(GOConfigReader &cfg) {
     buffer.Printf(wxT("SetterCrescendo%d"), i);
 
     bool defaultAddMode
-      = cfg.ReadBoolean(ODFSetting, buffer, OVERRIDE_MODE, false, true);
+      = cfg.ReadBoolean(ODFSetting, buffer, WX_OVERRIDE_MODE, false, true);
 
     m_CrescendoOverrideMode[i] = cfg.ReadBoolean(
-      CMBSetting, buffer, OVERRIDE_MODE, false, defaultAddMode);
+      CMBSetting, buffer, WX_OVERRIDE_MODE, false, defaultAddMode);
   }
   for (unsigned i = 0; i < N_CRESCENDOS * CRESCENDO_STEPS; i++) {
     m_crescendo.push_back(new GOGeneralCombination(
-      m_OrganController->GetGeneralTemplate(), m_OrganController, true));
+      *m_OrganController, m_OrganController->GetGeneralTemplate(), true));
     m_CrescendoExtraSets.emplace_back();
     buffer.Printf(
       wxT("SetterCrescendo%d_%03d"),
@@ -369,6 +408,15 @@ void GOSetter::Load(GOConfigReader &cfg) {
   m_buttons[ID_SETTER_INSERT]->Init(cfg, wxT("SetterInsert"), _("Insert"));
   m_buttons[ID_SETTER_DELETE]->Init(cfg, wxT("SetterDelete"), _("Delete"));
 
+  m_buttons[ID_SETTER_REFRESH_FILES]->Init(
+    cfg, GROUP_REFRESH_FILES, _("Refresh files"));
+  m_buttons[ID_SETTER_PREV_FILE]->Init(cfg, GROUP_PREV_FILE, _("Prev file"));
+  m_buttons[ID_SETTER_NEXT_FILE]->Init(cfg, GROUP_NEXT_FILE, _("Next file"));
+  m_buttons[ID_SETTER_LOAD_FILE]->Init(cfg, GROUP_LOAD_FILE, _("Load file"));
+  m_buttons[ID_SETTER_SAVE_FILE]->Init(cfg, GROUP_SAVE_FILE, _("Save file"));
+  m_buttons[ID_SETTER_SAVE_SETTINGS]->Init(
+    cfg, GROUP_SAVE_SETTINGS, _("Save settings"));
+
   m_buttons[ID_SETTER_CRESCENDO_PREV]->Init(
     cfg, wxT("SetterCrescendoPrev"), _("<"));
   m_buttons[ID_SETTER_CRESCENDO_NEXT]->Init(
@@ -399,7 +447,6 @@ void GOSetter::Load(GOConfigReader &cfg) {
   m_buttons[ID_SETTER_TRANSPOSE_UP]->Init(
     cfg, wxT("SetterTransposeUp"), _("+"));
 
-  m_buttons[ID_SETTER_SAVE]->Init(cfg, wxT("SetterSave"), _("Save"));
   m_buttons[ID_SETTER_ON]->Init(cfg, wxT("SetterOn"), _("ON"));
   m_buttons[ID_SETTER_ON]->Display(true);
 
@@ -411,6 +458,7 @@ void GOSetter::Load(GOConfigReader &cfg) {
     cfg, wxT("SetterCrescendoPosition"), _("crescendo position"));
   m_TransposeDisplay.Init(cfg, wxT("SetterTranspose"), _("transpose"));
   m_NameDisplay.Init(cfg, wxT("SetterName"), _("organ name"));
+  m_CurrFileDisplay.Init(cfg, KEY_CURR_FILE_NAME, _("current file name"));
 
   for (unsigned i = 0; i < 10; i++) {
     wxString group;
@@ -433,27 +481,235 @@ void GOSetter::Load(GOConfigReader &cfg) {
     cfg, wxT("SetterGeneralNext"), _("Next"));
 }
 
+void GOSetter::DisplayCmbFile(const wxString &fileName) {
+  const bool isValid = !fileName.IsEmpty();
+  const bool isTheSameAsLoaded = fileName == m_CmbFileLastLoaded;
+
+  m_CmbFileDisplayed = fileName;
+  m_CurrFileDisplay.SetContent(
+    isValid ? wxFileName(fileName).GetName() : wxString());
+  m_buttons[ID_SETTER_LOAD_FILE]->Display(isValid && !isTheSameAsLoaded);
+  m_buttons[ID_SETTER_SAVE_FILE]->Display(
+    isValid && isTheSameAsLoaded && m_state.m_IsModified);
+}
+
+int GOSetter::FindCmbFilePosFor(const wxString &yamlFile) {
+  return yamlFile.IsEmpty() ? -1 : m_CmbFileList.Index(yamlFile);
+}
+
+void GOSetter::MoveToCmbFile(int offset) {
+  if (!m_IsCmbFileListPopulated) {
+    // read the list of combination files
+    wxDir cmbDir(m_CmbFilesDir);
+
+    m_CmbFileList.Clear();
+    wxDir::GetAllFiles(m_CmbFilesDir, &m_CmbFileList, wxT("*.yaml"));
+    m_CmbFileList.Sort();
+    m_CmbFilePos = FindCmbFilePosFor(m_CmbFileDisplayed);
+    m_IsCmbFileListPopulated = true;
+  }
+
+  unsigned l = m_CmbFileList.GetCount();
+
+  if (l) {
+    assert(abs(offset) <= l);
+
+    // move along the list
+    if (m_CmbFilePos < 0 && offset < 0)
+      m_CmbFilePos = 0;                             // move from the end
+    m_CmbFilePos = (m_CmbFilePos + offset + l) % l; // wrap around the margins
+    DisplayCmbFile(m_CmbFileList[m_CmbFilePos]);
+  }
+}
+
+void GOSetter::NotifyCmbChanged() {
+  // Temporary we mark the organ modified when a combination is changed for
+  // the user would save the preset.
+  // But we intend to split combinations and organ settings, so in the future
+  // we will only mark the combinations as modified, not the organ settings
+  m_OrganController->SetOrganModified();
+}
+
+void GOSetter::NotifyCmbPushed(bool isChanged) {
+  if (isChanged && m_state.m_IsActive && !m_state.m_IsModified) {
+    m_state.m_IsModified = true;
+    // light the save button if the last loaded combination file  is displayed
+    if (
+      !m_CmbFileDisplayed.IsEmpty()
+      && m_CmbFileDisplayed == m_CmbFileLastLoaded)
+      m_buttons[ID_SETTER_SAVE_FILE]->Display(true);
+    NotifyCmbChanged();
+  }
+}
+
 void GOSetter::Save(GOConfigWriter &cfg) {
   for (unsigned i = 0; i < N_CRESCENDOS; i++) {
     cfg.WriteBoolean(
       wxString::Format(wxT("SetterCrescendo%d"), i),
-      OVERRIDE_MODE,
+      WX_OVERRIDE_MODE,
       m_CrescendoOverrideMode[i]);
   }
   // another objects are saveble themself so they are saved separatelly
 }
 
-void GOSetter::ButtonStateChanged(int id) {
+const char *const CURRENT = "current";
+const char *const SIMPLE_GENERALS = "generals";
+const char *const BANKED_GENERALS = "banked-generals";
+const char *const CRESCENDOS = "crescendos";
+const char *const OVERRIDE_MODE = "override-mode";
+const char *const STEPS = "steps";
+const char *const SEQUENCER = "sequencer";
+const wxString WX_C = wxT("%c");
+const wxString WX_C02U = wxT("%c%02u");
+const wxString WX_U = wxT("%u");
+const wxString WX_03U = wxT("%03u");
+
+wxString general_yaml_key(unsigned i) { return wxString::Format(WX_U, i); }
+
+wxString banked_general_yaml_key(unsigned i) {
+  return wxString::Format(WX_C02U, i / GENERALS + 'A', i % GENERALS + 1);
+}
+
+wxString crescendo_yaml_key(unsigned i) {
+  return wxString::Format(WX_C, i + 'A');
+}
+
+wxString crescendo_step_yaml_key(unsigned i) {
+  return wxString::Format(WX_U, i + 1);
+}
+
+wxString sequencer_cmb_yaml_key(unsigned i) {
+  return wxString::Format(WX_03U, i);
+}
+
+void GOSetter::ToYaml(YAML::Node &yamlNode) const {
+  // save generals
+  YAML::Node generalsNode;
+
+  for (unsigned l = m_OrganController->GetGeneralCount(), i = 0; i < l; i++)
+    m_OrganController->GetGeneral(i)->GetCombination().PutToYamlMap(
+      generalsNode, general_yaml_key(i));
+  put_to_map_if_not_null(yamlNode, SIMPLE_GENERALS, generalsNode);
+
+  // save banked generals
+  YAML::Node bankedGeneralsNode;
+
+  for (unsigned l = m_general.size(), i = 0; i < l; i++)
+    GOCombination::putToYamlMap(
+      bankedGeneralsNode, banked_general_yaml_key(i), m_general[i]);
+  put_to_map_if_not_null(yamlNode, BANKED_GENERALS, bankedGeneralsNode);
+
+  // save crescendos
+  YAML::Node crescendosNode;
+
+  for (unsigned i = 0; i < N_CRESCENDOS; i++) {
+    YAML::Node crescendoSteps;
+    unsigned baseIndex = CRESCENDO_STEPS * i;
+
+    for (unsigned j = 0; j < CRESCENDO_STEPS; j++)
+      GOCombination::putToYamlMap(
+        crescendoSteps, crescendo_step_yaml_key(j), m_crescendo[baseIndex + j]);
+    if (!crescendoSteps.IsNull()) {
+      YAML::Node crescendoNode = crescendosNode[crescendo_yaml_key(i)];
+
+      crescendoNode[OVERRIDE_MODE] = m_CrescendoOverrideMode[i];
+      crescendoNode[STEPS] = crescendoSteps;
+    }
+  }
+  put_to_map_if_not_null(yamlNode, CRESCENDOS, crescendosNode);
+
+  // save sequencer
+  YAML::Node sequencerNode;
+
+  for (unsigned i = 0; i < FRAME_GENERALS; i++)
+    GOCombination::putToYamlMap(
+      sequencerNode, wxString::Format(WX_03U, i), m_framegeneral[i]);
+  put_to_map_if_not_null(yamlNode, SEQUENCER, sequencerNode);
+}
+
+void GOSetter::FromYaml(const YAML::Node &yamlNode) {
+  // restore generals
+  const YAML::Node generalsNode = yamlNode[SIMPLE_GENERALS];
+
+  for (unsigned l = m_OrganController->GetGeneralCount(), i = 0; i < l; i++)
+    get_from_map_or_null(generalsNode, general_yaml_key(i))
+      >> m_OrganController->GetGeneral(i)->GetCombination();
+
+  // restore banked generals
+  const YAML::Node bankedGeneralsNode = yamlNode[BANKED_GENERALS];
+
+  for (unsigned l = m_general.size(), i = 0; i < l; i++)
+    get_from_map_or_null(bankedGeneralsNode, banked_general_yaml_key(i))
+      >> *m_general[i];
+
+  // restore crescendos
+  const YAML::Node crescendosNode = yamlNode[CRESCENDOS];
+
+  for (unsigned i = 0; i < N_CRESCENDOS; i++) {
+    const YAML::Node crescendoNode
+      = get_from_map_or_null(crescendosNode, crescendo_yaml_key(i));
+    const YAML::Node crescendoOverridNode
+      = get_from_map_or_null(crescendoNode, OVERRIDE_MODE);
+    const YAML::Node crescendoSteps
+      = get_from_map_or_null(crescendoNode, STEPS);
+    unsigned baseIndex = CRESCENDO_STEPS * i;
+
+    m_CrescendoOverrideMode[i] = crescendoOverridNode.as<bool, bool>(true);
+    for (unsigned j = 0; j < CRESCENDO_STEPS; j++)
+      get_from_map_or_null(crescendoSteps, crescendo_step_yaml_key(j))
+        >> *m_crescendo[baseIndex + j];
+  }
+
+  // restore sequencer
+  const YAML::Node sequencerNode = yamlNode[SEQUENCER];
+
+  for (unsigned i = 0; i < FRAME_GENERALS; i++)
+    get_from_map_or_null(sequencerNode, sequencer_cmb_yaml_key(i))
+      >> *m_framegeneral[i];
+}
+
+void GOSetter::ButtonStateChanged(int id, bool newState) {
   GOCombination::ExtraElementsSet elementSet;
 
   switch (id) {
+
+  case ID_SETTER_REFRESH_FILES:
+    m_IsCmbFileListPopulated = false;
+    break;
+  case ID_SETTER_PREV_FILE:
+    MoveToCmbFile(-1);
+    break;
+  case ID_SETTER_NEXT_FILE:
+    MoveToCmbFile(1);
+    break;
+  case ID_SETTER_LOAD_FILE:
+    if (!m_CmbFileDisplayed.IsEmpty()) {
+      // Flash the button until OnCombinationsLoaded()
+      m_buttons[ID_SETTER_LOAD_FILE]->Display(true);
+      m_OrganController->LoadCombination(m_CmbFileDisplayed);
+      // for minimising buttons to press we just make GrandOrgue ready to play
+      // from the home position
+      SetterActive(false);
+      SetPosition(0, true);
+    }
+    break;
+  case ID_SETTER_SAVE_FILE:
+    if (
+      !m_CmbFileLastLoaded.IsEmpty()
+      && m_CmbFileDisplayed == m_CmbFileLastLoaded)
+      m_OrganController->ExportCombination(m_CmbFileLastLoaded);
+    break;
   case ID_SETTER_PREV:
     Prev();
     break;
   case ID_SETTER_NEXT:
     Next();
     break;
+  case ID_SETTER_FULL:
+    m_state.m_IsStoreInvisible = newState;
+    break;
   case ID_SETTER_SET:
+    m_state.m_IsActive = newState;
     wxTheApp->GetTopWindow()->UpdateWindowUI();
     break;
   case ID_SETTER_M1:
@@ -491,12 +747,14 @@ void GOSetter::ButtonStateChanged(int id) {
   case ID_SETTER_DELETE:
     for (unsigned j = m_pos; j < m_framegeneral.size() - 1; j++)
       m_framegeneral[j]->Copy(m_framegeneral[j + 1]);
-    ResetDisplay();
+    UpdateAllButtonsLight(nullptr, -1);
+    NotifyCmbChanged();
     break;
   case ID_SETTER_INSERT:
     for (unsigned j = m_framegeneral.size() - 1; j > m_pos; j--)
       m_framegeneral[j]->Copy(m_framegeneral[j - 1]);
     SetPosition(m_pos);
+    NotifyCmbChanged();
     break;
   case ID_SETTER_L0:
   case ID_SETTER_L1:
@@ -560,10 +818,8 @@ void GOSetter::ButtonStateChanged(int id) {
   case ID_SETTER_GENERAL47:
   case ID_SETTER_GENERAL48:
   case ID_SETTER_GENERAL49:
-    m_general[id - ID_SETTER_GENERAL00 + m_bank * GENERALS]->Push(
-      GetCrescendoAddSet(elementSet));
-    ResetDisplay();
-    m_buttons[id]->Display(true);
+    PushGeneral(
+      *m_general[id - ID_SETTER_GENERAL00 + m_bank * GENERALS], m_buttons[id]);
     break;
   case ID_SETTER_GENERAL_PREV:
   case ID_SETTER_GENERAL_NEXT:
@@ -576,13 +832,13 @@ void GOSetter::ButtonStateChanged(int id) {
     break;
 
   case ID_SETTER_REGULAR:
-    SetSetterType(SETTER_REGULAR);
+    SetSetterType(GOSetterState::SETTER_REGULAR);
     break;
   case ID_SETTER_SCOPE:
-    SetSetterType(SETTER_SCOPE);
+    SetSetterType(GOSetterState::SETTER_SCOPE);
     break;
   case ID_SETTER_SCOPED:
-    SetSetterType(SETTER_SCOPED);
+    SetSetterType(GOSetterState::SETTER_SCOPED);
     break;
   case ID_SETTER_CRESCENDO_A:
   case ID_SETTER_CRESCENDO_B:
@@ -598,36 +854,40 @@ void GOSetter::ButtonStateChanged(int id) {
     Crescendo(m_crescendopos + 1, true);
     break;
   case ID_SETTER_CRESCENDO_CURRENT:
-    m_crescendo[m_crescendopos + m_crescendobank * CRESCENDO_STEPS]->Push();
+    PushGeneral(
+      *m_crescendo[m_crescendopos + m_crescendobank * CRESCENDO_STEPS],
+      nullptr);
     break;
 
-  case ID_SETTER_CRESCENDO_OVERRIDE: {
-    GOButtonControl *btn = m_buttons[ID_SETTER_CRESCENDO_OVERRIDE];
-    bool newIsOverride = !btn->IsEngaged();
-
-    m_CrescendoOverrideMode[m_crescendobank] = newIsOverride;
-    btn->Display(newIsOverride);
-  } break;
+  case ID_SETTER_CRESCENDO_OVERRIDE:
+    m_CrescendoOverrideMode[m_crescendobank] = newState;
+    m_buttons[ID_SETTER_CRESCENDO_OVERRIDE]->Display(newState);
 
   case ID_SETTER_PITCH_M1:
-    m_OrganController->GetRootPipeConfigNode().ModifyTuning(-1);
+    m_OrganController->GetRootPipeConfigNode().ModifyManualTuning(-1);
+    m_OrganController->GetRootPipeConfigNode().ModifyAutoTuningCorrection(-1);
     break;
   case ID_SETTER_PITCH_M10:
-    m_OrganController->GetRootPipeConfigNode().ModifyTuning(-10);
+    m_OrganController->GetRootPipeConfigNode().ModifyManualTuning(-10);
+    m_OrganController->GetRootPipeConfigNode().ModifyAutoTuningCorrection(-10);
     break;
   case ID_SETTER_PITCH_M100:
-    m_OrganController->GetRootPipeConfigNode().ModifyTuning(-100);
+    m_OrganController->GetRootPipeConfigNode().ModifyManualTuning(-100);
+    m_OrganController->GetRootPipeConfigNode().ModifyAutoTuningCorrection(-100);
     break;
   case ID_SETTER_PITCH_P1:
-    m_OrganController->GetRootPipeConfigNode().ModifyTuning(1);
+    m_OrganController->GetRootPipeConfigNode().ModifyManualTuning(1);
+    m_OrganController->GetRootPipeConfigNode().ModifyAutoTuningCorrection(1);
     break;
   case ID_SETTER_PITCH_P10:
-    m_OrganController->GetRootPipeConfigNode().ModifyTuning(10);
+    m_OrganController->GetRootPipeConfigNode().ModifyManualTuning(10);
+    m_OrganController->GetRootPipeConfigNode().ModifyAutoTuningCorrection(10);
     break;
   case ID_SETTER_PITCH_P100:
-    m_OrganController->GetRootPipeConfigNode().ModifyTuning(100);
+    m_OrganController->GetRootPipeConfigNode().ModifyManualTuning(100);
+    m_OrganController->GetRootPipeConfigNode().ModifyAutoTuningCorrection(100);
     break;
-  case ID_SETTER_SAVE:
+  case ID_SETTER_SAVE_SETTINGS:
     m_OrganController->Save();
     break;
 
@@ -664,8 +924,6 @@ void GOSetter::ButtonStateChanged(int id) {
   }
 }
 
-void GOSetter::AbortPlayback() {}
-
 void GOSetter::PreparePlayback() {
   wxString buffer;
   buffer.Printf(wxT("%03d"), m_pos);
@@ -686,24 +944,119 @@ void GOSetter::PreparePlayback() {
   UpdateTranspose();
 }
 
-void GOSetter::StartPlayback() {}
+void GOSetter::OnCombinationsLoaded(
+  const wxString &yamlDir, const wxString &yamlFile) {
+  if (yamlDir != m_CmbFilesDir) {
+    m_CmbFilesDir = yamlDir;
+    m_CmbFileList.clear();
+    m_IsCmbFileListPopulated = false;
+    m_CmbFilePos = -1;
+    // loaded combinations from a new dir
+  } else {
+    // loaded combinations from the same dir as before
+    m_CmbFilePos = m_IsCmbFileListPopulated && !yamlFile.IsEmpty()
+      ? FindCmbFilePosFor(yamlFile)
+      : -1;
+  }
+  m_CmbFileLastLoaded = yamlFile;
+  m_state.m_IsModified = false;
+  DisplayCmbFile(m_CmbFileLastLoaded);
+}
 
-void GOSetter::PrepareRecording() {}
+void GOSetter::OnCombinationsSaved(const wxString &yamlFile) {
+  if (yamlFile == m_CmbFileLastLoaded)
+    m_state.m_IsModified = false;
+  else if (wxFileName(yamlFile).GetPath() == m_CmbFilesDir) {
+    // Possible a new file has been created in the same directory, so we need to
+    // refresh
+    m_IsCmbFileListPopulated = false;
+    // switch to the new file
+    m_CmbFileLastLoaded = yamlFile;
+    m_state.m_IsModified = false;
+  }
+  DisplayCmbFile(m_CmbFileLastLoaded);
+}
 
 void GOSetter::Update() {}
 
-bool GOSetter::IsSetterActive() {
-  return m_buttons[ID_SETTER_SET]->IsEngaged();
-}
-
-bool GOSetter::StoreInvisibleObjects() {
-  return m_buttons[ID_SETTER_FULL]->IsEngaged();
-}
-
 void GOSetter::SetterActive(bool on) { m_buttons[ID_SETTER_SET]->Set(on); }
 
 void GOSetter::ToggleSetter() { m_buttons[ID_SETTER_SET]->Push(); }
 
+void GOSetter::UpdateAllButtonsLight(
+  GOButtonControl *buttonToLight, int manualIndexOnlyFor) {
+  if (manualIndexOnlyFor < 0) {
+    updateOneButtonLight(m_buttons[ID_SETTER_HOME], buttonToLight);
+    for (unsigned i = 0; i < 10; i++)
+      updateOneButtonLight(m_buttons[ID_SETTER_L0 + i], buttonToLight);
+    for (unsigned i = 0; i < GENERALS; i++)
+      updateOneButtonLight(m_buttons[ID_SETTER_GENERAL00 + i], buttonToLight);
+  }
+}
+
+void GOSetter::UpdateAllSetsButtonsLight(
+  GOButtonControl *buttonToLight, int manualIndexOnlyFor) {
+  for (GOCombinationButtonSet *pCmbButtonSet :
+       m_OrganController->GetCombinationButtonSets())
+    pCmbButtonSet->UpdateAllButtonsLight(buttonToLight, manualIndexOnlyFor);
+}
+
+void GOSetter::PushGeneral(
+  GOGeneralCombination &cmb, GOButtonControl *pButtonToLight) {
+  GOCombination::ExtraElementsSet elementSet;
+  const GOCombination::ExtraElementsSet *pExtraSet
+    = GetCrescendoAddSet(elementSet);
+
+  NotifyCmbPushed(cmb.Push(m_state, pExtraSet));
+  if (!pExtraSet) { // Otherwise the crescendo in add mode:
+                    // not to switch off combination buttons
+    UpdateAllSetsButtonsLight(pButtonToLight, -1);
+  }
+}
+
+void GOSetter::PushDivisional(
+  GODivisionalCombination &cmb, GOButtonControl *pButtonToLight) {
+  GOCombination::ExtraElementsSet elementSet;
+  const GOCombination::ExtraElementsSet *pExtraSet
+    = GetCrescendoAddSet(elementSet);
+
+  NotifyCmbPushed(cmb.Push(m_state, pExtraSet));
+  /* only use divisional couples, if not in setter mode */
+  if (!m_state.m_IsActive) {
+    unsigned cmbManualNumber = cmb.GetManualNumber();
+    unsigned divisionalNumber = cmb.GetDivisionalNumber();
+
+    for (unsigned k = 0; k < m_OrganController->GetDivisionalCouplerCount();
+         k++) {
+      GODivisionalCoupler *coupler = m_OrganController->GetDivisionalCoupler(k);
+      if (!coupler->IsEngaged())
+        continue;
+
+      for (unsigned i = 0; i < coupler->GetNumberOfManuals(); i++) {
+        if (coupler->GetManual(i) != cmbManualNumber)
+          continue;
+
+        for (unsigned int j = i + 1; j < coupler->GetNumberOfManuals(); j++)
+          m_OrganController->GetManual(coupler->GetManual(j))
+            ->GetDivisional(divisionalNumber)
+            ->Push();
+
+        if (coupler->IsBidirectional())
+          for (unsigned j = 0; j < coupler->GetNumberOfManuals(); j++) {
+            if (coupler->GetManual(j) == cmbManualNumber)
+              break;
+            m_OrganController->GetManual(coupler->GetManual(j))
+              ->GetDivisional(divisionalNumber)
+              ->Push();
+          }
+        break;
+      }
+    }
+  }
+  if (!pExtraSet)
+    UpdateAllSetsButtonsLight(pButtonToLight, cmb.GetManualNumber());
+}
+
 void GOSetter::Next() { SetPosition(m_pos + 1); }
 
 void GOSetter::Prev() { SetPosition(m_pos - 1); }
@@ -712,13 +1065,11 @@ void GOSetter::Push() { SetPosition(m_pos); }
 
 unsigned GOSetter::GetPosition() { return m_pos; }
 
-SetterType GOSetter::GetSetterType() { return m_SetterType; }
-
-void GOSetter::SetSetterType(SetterType type) {
-  m_SetterType = type;
-  m_buttons[ID_SETTER_REGULAR]->Display(type == SETTER_REGULAR);
-  m_buttons[ID_SETTER_SCOPE]->Display(type == SETTER_SCOPE);
-  m_buttons[ID_SETTER_SCOPED]->Display(type == SETTER_SCOPED);
+void GOSetter::SetSetterType(GOSetterState::SetterType type) {
+  m_state.m_SetterType = type;
+  m_buttons[ID_SETTER_REGULAR]->Display(type == GOSetterState::SETTER_REGULAR);
+  m_buttons[ID_SETTER_SCOPE]->Display(type == GOSetterState::SETTER_SCOPE);
+  m_buttons[ID_SETTER_SCOPED]->Display(type == GOSetterState::SETTER_SCOPED);
 }
 
 void GOSetter::SetCrescendoType(unsigned no) {
@@ -743,14 +1094,6 @@ const GOCombination::ExtraElementsSet *GOSetter::GetCrescendoAddSet(
   return pResElementSet;
 }
 
-void GOSetter::ResetDisplay() {
-  m_buttons[ID_SETTER_HOME]->Display(false);
-  for (unsigned i = 0; i < 10; i++)
-    m_buttons[ID_SETTER_L0 + i]->Display(false);
-  for (unsigned i = 0; i < GENERALS; i++)
-    m_buttons[ID_SETTER_GENERAL00 + i]->Display(false);
-}
-
 void GOSetter::UpdatePosition(int pos) {
   if (pos != (int)m_pos)
     SetPosition(pos);
@@ -765,15 +1108,8 @@ void GOSetter::SetPosition(int pos, bool push) {
     pos -= m_framegeneral.size();
   m_pos = pos;
   if (push) {
-    GOCombination::ExtraElementsSet elementSet;
-
-    m_framegeneral[m_pos]->Push(GetCrescendoAddSet(elementSet));
-
+    PushGeneral(*m_framegeneral[m_pos], m_buttons[ID_SETTER_L0 + m_pos % 10]);
     m_buttons[ID_SETTER_HOME]->Display(m_pos == 0);
-    for (unsigned i = 0; i < 10; i++)
-      m_buttons[ID_SETTER_L0 + i]->Display((m_pos % 10) == i);
-    for (unsigned i = 0; i < GENERALS; i++)
-      m_buttons[ID_SETTER_GENERAL00 + i]->Display(false);
   }
 
   buffer.Printf(wxT("%03d"), m_pos);
@@ -790,13 +1126,14 @@ void GOSetter::Crescendo(int newpos, bool force) {
     newpos = 0;
   if (newpos > CRESCENDO_STEPS - 1)
     newpos = CRESCENDO_STEPS - 1;
-  if (IsSetterActive() && !force)
+  if (m_state.m_IsActive && !force)
     return;
   unsigned pos = newpos;
   if (pos == m_crescendopos)
     return;
 
   bool crescendoAddMode = !m_CrescendoOverrideMode[m_crescendobank];
+  bool changed = false;
 
   while (pos > m_crescendopos) {
     const unsigned oldIdx = m_crescendopos + m_crescendobank * CRESCENDO_STEPS;
@@ -807,8 +1144,9 @@ void GOSetter::Crescendo(int newpos, bool force) {
     else
       m_CrescendoExtraSets[oldIdx].clear();
     ++m_crescendopos;
-    m_crescendo[newIdx]->Push(
-      crescendoAddMode ? &m_CrescendoExtraSets[oldIdx] : nullptr, true);
+    changed = changed
+      || m_crescendo[newIdx]->Push(
+        m_state, crescendoAddMode ? &m_CrescendoExtraSets[oldIdx] : nullptr);
   }
 
   while (pos < m_crescendopos) {
@@ -816,9 +1154,14 @@ void GOSetter::Crescendo(int newpos, bool force) {
 
     const unsigned newIdx = m_crescendopos + m_crescendobank * CRESCENDO_STEPS;
 
-    m_crescendo[newIdx]->Push(
-      crescendoAddMode ? &m_CrescendoExtraSets[newIdx] : nullptr, true);
+    changed = changed
+      || m_crescendo[newIdx]->Push(
+        m_state, crescendoAddMode ? &m_CrescendoExtraSets[newIdx] : nullptr);
   }
+  // switch combination buttons off in the crescendo override mode
+  if (changed && !crescendoAddMode)
+    ResetCmbButtons();
+  NotifyCmbPushed(changed);
 
   wxString buffer;
   buffer.Printf(wxT("%d"), m_crescendopos + 1);
@@ -847,7 +1190,7 @@ void GOSetter::SetTranspose(int value) {
 }
 
 void GOSetter::UpdateModified(bool modified) {
-  m_buttons[ID_SETTER_SAVE]->Display(modified);
+  m_buttons[ID_SETTER_SAVE_SETTINGS]->Display(modified);
 }
 
 GOEnclosure *GOSetter::GetEnclosure(const wxString &name, bool is_panel) {
@@ -876,6 +1219,9 @@ GOLabelControl *GOSetter::GetLabelControl(const wxString &name, bool is_panel) {
   if (name == wxT("TransposeLabel"))
     return &m_TransposeDisplay;
 
+  if (name == KEY_CURR_FILE_NAME)
+    return &m_CurrFileDisplay;
+
   if (is_panel)
     return NULL;
 
