@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2022 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -12,10 +12,12 @@
 #include "control/GOPushbuttonControl.h"
 
 class GOConfigReader;
+class GOSetter;
 
 class GODivisionalButtonControl : public GOPushbuttonControl {
 private:
-  GODivisionalCombination m_divisional;
+  GOSetter &r_setter;
+  GODivisionalCombination m_combination;
 
 public:
   GODivisionalButtonControl(
@@ -23,7 +25,7 @@ public:
     GOCombinationDefinition &divisionalTemplate,
     bool isSetter);
 
-  GODivisionalCombination &GetGeneral() { return m_divisional; }
+  GODivisionalCombination &GetCombination() { return m_combination; }
   wxString GetMidiType() override;
 
   void Init(
@@ -42,7 +44,7 @@ public:
   void LoadCombination(GOConfigReader &cfg);
   void Save(GOConfigWriter &cfg);
 
-  void Push();
+  void Push() override;
 };
 
 #endif /* GODIVISIONALBUTTONCONTROL_H */
