@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2022 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -17,21 +17,21 @@ GOGeneralButtonControl::GOGeneralButtonControl(
   GOCombinationDefinition &general_template,
   GOOrganController *organController,
   bool is_setter)
-  : GOPushbuttonControl(organController),
-    m_general(general_template, organController, is_setter) {}
+  : GOPushbuttonControl(*organController),
+    r_setter(*organController->GetSetter()),
+    m_combination(*organController, general_template, is_setter) {}
 
 void GOGeneralButtonControl::Load(GOConfigReader &cfg, wxString group) {
-  m_general.Load(cfg, group);
+  m_combination.Load(cfg, group);
   GOPushbuttonControl::Load(cfg, group);
 }
 
 void GOGeneralButtonControl::Push() {
-  GOCombination::ExtraElementsSet elementSet;
-
-  m_general.Push(
-    m_OrganController->GetSetter()->GetCrescendoAddSet(elementSet));
+  r_setter.PushGeneral(m_combination, this);
 }
 
-GOGeneralCombination &GOGeneralButtonControl::GetGeneral() { return m_general; }
+GOGeneralCombination &GOGeneralButtonControl::GetCombination() {
+  return m_combination;
+}
 
 wxString GOGeneralButtonControl::GetMidiType() { return _("General"); }
