@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2022 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -12,10 +12,12 @@
 #include "control/GOPushbuttonControl.h"
 
 class GOConfigReader;
+class GOSetter;
 
 class GOGeneralButtonControl : public GOPushbuttonControl {
 private:
-  GOGeneralCombination m_general;
+  GOSetter &r_setter;
+  GOGeneralCombination m_combination;
 
 public:
   GOGeneralButtonControl(
@@ -23,8 +25,8 @@ public:
     GOOrganController *organController,
     bool is_setter);
   void Load(GOConfigReader &cfg, wxString group);
-  void Push();
-  GOGeneralCombination &GetGeneral();
+  void Push() override;
+  GOGeneralCombination &GetCombination();
 
   wxString GetMidiType();
 };
