@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2022 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -36,12 +36,18 @@ private:
   wxSpinButton *m_AmplitudeSpin;
   wxTextCtrl *m_Gain;
   wxSpinButton *m_GainSpin;
-  wxTextCtrl *m_Tuning;
-  wxSpinButton *m_TuningSpin;
+  wxTextCtrl *m_ManualTuning;
+  wxSpinButton *m_ManualTuningSpin;
+  wxTextCtrl *m_AutoTuningCorrection;
+  wxSpinButton *m_AutoTuningCorrectionSpin;
   wxTextCtrl *m_Delay;
   wxSpinButton *m_DelaySpin;
+  wxTextCtrl *m_ReleaseLength;
+  wxSpinButton *m_ReleaseLengthSpin;
   wxComboBox *m_AudioGroup;
   wxString m_LastAudioGroup;
+  wxCheckBox *m_IgnorePitch;
+  bool m_LastIgnorePitch;
   wxChoice *m_BitsPerSample;
   int m_LastBitsPerSample;
   wxChoice *m_Channels;
@@ -62,7 +68,6 @@ private:
   wxButton *m_DefaultAll;
   wxButton *m_AudioGroupAssistant;
   wxButton *m_Collapse;
-  wxCheckBox *m_IgnorePitch;
   OrganTreeItemData *m_Last;
   unsigned m_LoadChangeCnt;
   wxDialog *m_ModalDialog;
@@ -89,11 +94,16 @@ private:
   void OnAmplitudeChanged(wxCommandEvent &e);
   void OnGainSpinChanged(wxSpinEvent &e);
   void OnGainChanged(wxCommandEvent &e);
-  void OnTuningSpinChanged(wxSpinEvent &e);
-  void OnTuningChanged(wxCommandEvent &e);
+  void OnManualTuningSpinChanged(wxSpinEvent &e);
+  void OnManualTuningChanged(wxCommandEvent &e);
+  void OnAutoTuningCorrectionSpinChanged(wxSpinEvent &e);
+  void OnAutoTuningCorrectionChanged(wxCommandEvent &e);
   void OnDelaySpinChanged(wxSpinEvent &e);
   void OnDelayChanged(wxCommandEvent &e);
+  void OnReleaseLengthSpinChanged(wxSpinEvent &e);
+  void OnReleaseLengthChanged(wxCommandEvent &e);
   void OnAudioGroupChanged(wxCommandEvent &e);
+  void OnIgnorePitchChanged(wxCommandEvent &e);
   void OnBitsPerSampleChanged(wxCommandEvent &e);
   void OnCompressChanged(wxCommandEvent &e);
   void OnChannelsChanged(wxCommandEvent &e);
@@ -122,10 +132,14 @@ protected:
     ID_EVENT_AMPLITUDE_SPIN,
     ID_EVENT_GAIN,
     ID_EVENT_GAIN_SPIN,
-    ID_EVENT_TUNING,
-    ID_EVENT_TUNING_SPIN,
+    ID_EVENT_MANUAL_TUNING,
+    ID_EVENT_MANUAL_TUNING_SPIN,
+    ID_EVENT_AUTO_TUNING_CORRECTION,
+    ID_EVENT_AUTO_TUNING_CORRECTION_SPIN,
     ID_EVENT_DELAY,
     ID_EVENT_DELAY_SPIN,
+    ID_EVENT_RELEASE_LENGTH,
+    ID_EVENT_RELEASE_LENGTH_SPIN,
     ID_EVENT_AUDIO_GROUP,
     ID_EVENT_IGNORE_PITCH,
     ID_EVENT_LOOP_LOAD,
