@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2022 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -11,8 +11,8 @@
 
 #include "config/GOConfigReader.h"
 #include "control/GOButtonControl.h"
+#include "primitives/GODC.h"
 
-#include "GODC.h"
 #include "GOGUIDisplayMetrics.h"
 #include "GOGUILayoutEngine.h"
 #include "GOGUIMouseState.h"
@@ -171,7 +171,7 @@ void GOGUIButton::Load(GOConfigReader &cfg, wxString group) {
       group,
       wxT("DispImageNum"),
       1,
-      6,
+      7,
       false,
       m_ButtonControl->IsReadOnly() ? 4 : 1);
     off_file
