@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2022 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -10,8 +10,9 @@
 #include "config/GOConfig.h"
 #include "config/GOConfigReader.h"
 #include "control/GOLabelControl.h"
+#include "primitives/GOBitmap.h"
+#include "primitives/GODC.h"
 
-#include "GODC.h"
 #include "GOGUIDisplayMetrics.h"
 #include "GOGUILayoutEngine.h"
 #include "GOGUIPanel.h"
@@ -23,9 +24,7 @@ GOGUILabel::GOGUILabel(GOGUIPanel *panel, GOLabelControl *label)
     m_DispXpos(0),
     m_DispYpos(0),
     m_Label(label),
-    m_Bitmap(),
-    m_FontSize(0),
-    m_FontName(),
+    m_PBackgroundBitmap(nullptr),
     m_Text(),
     m_TextColor(0, 0, 0),
     m_TextRect(),
@@ -33,48 +32,96 @@ GOGUILabel::GOGUILabel(GOGUIPanel *panel, GOLabelControl *label)
     m_TileOffsetX(0),
     m_TileOffsetY(0) {}
 
+/**
+ * Deletes m_PBackgroundBitmap if it has been allocated
+ */
+void GOGUILabel::DestroyBackgroundBitmap() {
+  GOBitmap *pOldBackgroundBitmap = m_PBackgroundBitmap;
+
+  // reset the old background
+  if (pOldBackgroundBitmap) {
+    m_PBackgroundBitmap = nullptr;
+    delete pOldBackgroundBitmap;
+  }
+}
+
+static const wxString WX_BITMAP_LABEL_FMT = wxT(GOBitmapPrefix "label%02d");
+
+/**
+ * Creates m_PBackgroundBitmap image from the imageFileName and imageNum and
+ * sets the default values for m_BoundingRect, m_TextRect, m_TextWidth,
+ * m_TileOffsetX, m_TileOffsetY, basing on the image size
+ * @param x - the x coordinate for m_BoundingRect
+ * @param x - the y coordinate for m_BoundingRect
+ * @param imageFileName the file name of the bitmap. If it is empty then
+ *   it is calculated from imageNum
+ * @param imageNum if > 0 then the number of predefined images. If 0 then the
+ *   background is transparent. This parameter is used only if imageFileName is
+ *   empty.
+ * @param imageMaskFilename a filename of the image mask
+ */
+void GOGUILabel::InitBackgroundBitmap(
+  unsigned x,
+  unsigned y,
+  unsigned w,
+  unsigned h,
+  wxString imageFileName,
+  unsigned imageNum,
+  const wxString &imageMaskFilename) {
+  DestroyBackgroundBitmap();
+  if (imageFileName.IsEmpty() && imageNum)
+    // Calculate imageFileName from imageNum
+    imageFileName.Printf(WX_BITMAP_LABEL_FMT, imageNum);
+  if (!imageFileName.IsEmpty())
+    m_PBackgroundBitmap
+      = new GOBitmap(m_panel->LoadBitmap(imageFileName, imageMaskFilename));
+
+  // take the size from the bitmap if it is not specified
+  if (!w)
+    w = m_PBackgroundBitmap ? m_PBackgroundBitmap->GetWidth() : 80;
+  if (!h)
+    h = m_PBackgroundBitmap ? m_PBackgroundBitmap->GetHeight() : 25;
+  unsigned textX = 1;
+  unsigned textY = 1;
+  unsigned textWidth = w - textX;
+  unsigned textHeigth = h - textY;
+
+  m_BoundingRect = wxRect(x, y, w, h);
+  m_TextRect = wxRect(textX, textY, textWidth, textHeigth);
+  m_TextWidth = textWidth;
+  m_TileOffsetX = 0;
+  m_TileOffsetY = 0;
+}
+
+void GOGUILabel::InitFont(const wxString &fontName, unsigned fontSize) {
+  m_Font = m_metrics->GetGroupLabelFont();
+  SetFontName(fontName);
+  SetFontSize(fontSize);
+}
+
 void GOGUILabel::Init(
   GOConfigReader &cfg,
   wxString group,
   unsigned x_pos,
   unsigned y_pos,
   wxString name,
-  unsigned DispImageNum) {
+  unsigned DispImageNum,
+  unsigned w,
+  unsigned h) {
   GOGUIControl::Init(cfg, group);
 
   m_TextColor = wxColour(0x00, 0x00, 0x00);
-  m_FontSize = 7;
-  m_FontName = wxT("");
   m_Text = name;
 
-  wxString image_file
-    = wxString::Format(wxT(GOBitmapPrefix "label%02d"), DispImageNum);
-  wxString image_mask_file = wxEmptyString;
-
-  m_Bitmap = m_panel->LoadBitmap(image_file, image_mask_file);
-
-  int x, y, w, h;
-  x = x_pos;
-  y = y_pos;
-  w = m_Bitmap.GetWidth();
-  h = m_Bitmap.GetHeight();
-  m_BoundingRect = wxRect(x, y, w, h);
-
-  m_TileOffsetX = 0;
-  m_TileOffsetY = 0;
-
-  x = 1;
-  y = 1;
-  w = m_BoundingRect.GetWidth() - x;
-  h = m_BoundingRect.GetHeight() - y;
-  m_TextRect = wxRect(x, y, w, h);
-  m_TextWidth = m_TextRect.GetWidth();
-
-  m_Font = m_metrics->GetGroupLabelFont();
-  m_Font.SetName(m_FontName);
-  m_Font.SetPoints(m_FontSize);
+  InitBackgroundBitmap(
+    x_pos, y_pos, w, h, wxEmptyString, DispImageNum, wxEmptyString);
+  InitFont(wxEmptyString, 7);
 }
 
+static const wxString WX_IMAGE = wxT("Image");
+static const wxString WX_DISP_IMAGE_NUM = wxT("DispImageNum");
+static const wxString WX_MASK = wxT("Mask");
+
 void GOGUILabel::Load(GOConfigReader &cfg, wxString group) {
   GOGUIControl::Load(cfg, group);
 
@@ -134,31 +181,6 @@ void GOGUILabel::Load(GOConfigReader &cfg, wxString group) {
       0);
   }
 
-  m_TextColor = logicalToWxColour(cfg.ReadColor(
-    ODFSetting, group, wxT("DispLabelColour"), false, wxT("BLACK")));
-  m_FontSize = cfg.ReadFontSize(
-    ODFSetting, group, wxT("DispLabelFontSize"), false, wxT("normal"));
-  m_FontName = cfg.ReadStringTrim(
-    ODFSetting, group, wxT("DispLabelFontName"), false, wxT(""));
-  if (!m_Label || !m_panel->GetOrganFile()->GetSettings().ODFCheck())
-    m_Text
-      = cfg.ReadString(ODFSetting, group, wxT("Name"), false, wxEmptyString);
-
-  unsigned DispImageNum
-    = cfg.ReadInteger(ODFSetting, group, wxT("DispImageNum"), 1, 12, false, 1);
-
-  wxString image_file = cfg.ReadStringTrim(
-    ODFSetting,
-    group,
-    wxT("Image"),
-    false,
-    wxString::Format(wxT(GOBitmapPrefix "label%02d"), DispImageNum));
-  wxString image_mask_file
-    = cfg.ReadStringTrim(ODFSetting, group, wxT("Mask"), false, wxEmptyString);
-
-  m_Bitmap = m_panel->LoadBitmap(image_file, image_mask_file);
-
-  int w, h;
   x = cfg.ReadInteger(
     ODFSetting,
     group,
@@ -175,30 +197,47 @@ void GOGUILabel::Load(GOConfigReader &cfg, wxString group) {
     m_metrics->GetScreenHeight(),
     false,
     y);
-  w = cfg.ReadInteger(
-    ODFSetting,
-    group,
-    wxT("Width"),
-    1,
-    m_metrics->GetScreenWidth(),
-    false,
-    m_Bitmap.GetWidth());
-  h = cfg.ReadInteger(
-    ODFSetting,
-    group,
-    wxT("Height"),
-    1,
-    m_metrics->GetScreenHeight(),
-    false,
-    m_Bitmap.GetHeight());
-  m_BoundingRect = wxRect(x, y, w, h);
+
+  m_TextColor = logicalToWxColour(cfg.ReadColor(
+    ODFSetting, group, wxT("DispLabelColour"), false, wxT("BLACK")));
+  if (!m_Label || !m_panel->GetOrganFile()->GetSettings().ODFCheck())
+    m_Text
+      = cfg.ReadString(ODFSetting, group, wxT("Name"), false, wxEmptyString);
+
+  InitBackgroundBitmap(
+    x,
+    y,
+    cfg.ReadInteger(
+      ODFSetting,
+      group,
+      wxT("Width"),
+      1,
+      m_metrics->GetScreenWidth(),
+      false,
+      0),
+    cfg.ReadInteger(
+      ODFSetting,
+      group,
+      wxT("Height"),
+      1,
+      m_metrics->GetScreenHeight(),
+      false,
+      0),
+    cfg.ReadStringTrim(ODFSetting, group, WX_IMAGE, false),
+    cfg.ReadInteger(ODFSetting, group, WX_DISP_IMAGE_NUM, 0, 15, false, 1),
+    cfg.ReadStringTrim(ODFSetting, group, WX_MASK, false));
+  InitFont(
+    cfg.ReadStringTrim(
+      ODFSetting, group, wxT("DispLabelFontName"), false, wxEmptyString),
+    cfg.ReadFontSize(
+      ODFSetting, group, wxT("DispLabelFontSize"), false, wxT("normal")));
 
   m_TileOffsetX = cfg.ReadInteger(
     ODFSetting,
     group,
     wxT("TileOffsetX"),
     0,
-    m_Bitmap.GetWidth() - 1,
+    m_BoundingRect.width - 1,
     false,
     0);
   m_TileOffsetY = cfg.ReadInteger(
@@ -206,7 +245,7 @@ void GOGUILabel::Load(GOConfigReader &cfg, wxString group) {
     group,
     wxT("TileOffsetY"),
     0,
-    m_Bitmap.GetHeight() - 1,
+    m_BoundingRect.height - 1,
     false,
     0);
 
@@ -215,46 +254,37 @@ void GOGUILabel::Load(GOConfigReader &cfg, wxString group) {
     group,
     wxT("TextRectLeft"),
     0,
-    m_BoundingRect.GetWidth() - 1,
+    m_BoundingRect.width - 1,
     false,
-    1);
+    m_TextRect.x);
   y = cfg.ReadInteger(
     ODFSetting,
     group,
     wxT("TextRectTop"),
     0,
-    m_BoundingRect.GetHeight() - 1,
+    m_BoundingRect.height - 1,
     false,
-    1);
-  w = cfg.ReadInteger(
+    m_TextRect.y);
+
+  unsigned w = cfg.ReadInteger(
     ODFSetting,
     group,
     wxT("TextRectWidth"),
     1,
-    m_BoundingRect.GetWidth() - x,
+    m_BoundingRect.width - x,
     false,
-    m_BoundingRect.GetWidth() - x);
-  h = cfg.ReadInteger(
+    m_BoundingRect.width - x);
+  unsigned h = cfg.ReadInteger(
     ODFSetting,
     group,
     wxT("TextRectHeight"),
     1,
-    m_BoundingRect.GetHeight() - y,
+    m_BoundingRect.height - y,
     false,
-    m_BoundingRect.GetHeight() - y);
+    m_BoundingRect.height - y);
   m_TextRect = wxRect(x, y, w, h);
-  m_TextWidth = cfg.ReadInteger(
-    ODFSetting,
-    group,
-    wxT("TextBreakWidth"),
-    0,
-    m_TextRect.GetWidth(),
-    false,
-    m_TextRect.GetWidth());
-
-  m_Font = m_metrics->GetGroupLabelFont();
-  m_Font.SetName(m_FontName);
-  m_Font.SetPoints(m_FontSize);
+  m_TextWidth
+    = cfg.ReadInteger(ODFSetting, group, wxT("TextBreakWidth"), 0, w, false, w);
 }
 
 void GOGUILabel::Layout() {
@@ -278,15 +308,17 @@ void GOGUILabel::Layout() {
 }
 
 void GOGUILabel::PrepareDraw(double scale, GOBitmap *background) {
-  m_Bitmap.PrepareTileBitmap(
-    scale, m_BoundingRect, m_TileOffsetX, m_TileOffsetY, background);
+  if (m_PBackgroundBitmap)
+    m_PBackgroundBitmap->PrepareTileBitmap(
+      scale, m_BoundingRect, m_TileOffsetX, m_TileOffsetY, background);
 }
 
 void GOGUILabel::Draw(GODC &dc) {
   if (m_Label)
     m_Text = m_Label->GetContent();
 
-  dc.DrawBitmap(m_Bitmap, m_BoundingRect);
+  if (m_PBackgroundBitmap)
+    dc.DrawBitmap(*m_PBackgroundBitmap, m_BoundingRect);
   if (m_TextWidth)
     dc.DrawText(m_Text, m_TextRect, m_TextColor, m_Font, m_TextWidth);
 
