@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2022 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -10,7 +10,7 @@
 
 #include <wx/colour.h>
 
-#include "config/GOLogicalColour.h"
+#include "GOLogicalColour.h"
 
 inline wxColour logicalToWxColour(const GOLogicalColour &lc) {
   return wxColour(lc.m_red, lc.m_green, lc.m_blue);
