@@ -1,6 +1,6 @@
 /*
  * Copyright 2006 Milan Digital Audio LLC
- * Copyright 2009-2022 GrandOrgue contributors (see AUTHORS)
+ * Copyright 2009-2023 GrandOrgue contributors (see AUTHORS)
  * License GPL-2.0 or later
  * (https://www.gnu.org/licenses/old-licenses/gpl-2.0.html).
  */
@@ -10,17 +10,17 @@
 
 #include <wx/string.h>
 
-#include <vector>
+#include "ptrvector.h"
 
+#include "combinations/control/GOCombinationButtonSet.h"
 #include "combinations/model/GOCombinationDefinition.h"
-
-#include "GOEventHandler.h"
-#include "GOPlaybackStateHandler.h"
-#include "GOSaveableObject.h"
 #include "midi/GOMidiConfigurator.h"
 #include "midi/GOMidiReceiver.h"
 #include "midi/GOMidiSender.h"
-#include "ptrvector.h"
+#include "sound/GOSoundStateHandler.h"
+
+#include "GOEventHandler.h"
+#include "GOSaveableObject.h"
 
 class GOConfigReader;
 class GOConfigWriter;
@@ -33,8 +33,9 @@ class GOTremulant;
 class GOOrganController;
 
 class GOManual : private GOEventHandler,
+                 private GOCombinationButtonSet,
                  private GOSaveableObject,
-                 private GOPlaybackStateHandler,
+                 private GOSoundStateHandler,
                  public GOMidiConfigurator {
 private:
   wxString m_group;
@@ -82,9 +83,16 @@ private:
 
   void AbortPlayback();
   void PreparePlayback();
-  void StartPlayback();
   void PrepareRecording();
 
+  /**
+   * Update all divisional buttons light.
+   * @param buttonToLight
+   * @param manualIndexOnlyFor
+   */
+  void UpdateAllButtonsLight(
+    GOButtonControl *buttonToLight, int manualIndexOnlyFor) override;
+
 public:
   GOManual(GOOrganController *organController);
   void Init(
@@ -112,19 +120,46 @@ public:
   void AllNotesOff();
   bool IsKeyDown(unsigned midiNoteNumber);
 
-  unsigned GetStopCount();
+  unsigned GetStopCount() const { return m_stops.size(); };
   GOStop *GetStop(unsigned index);
-  unsigned GetCouplerCount();
-  unsigned GetODFCouplerCount();
+
+  /**
+   * Find a stop by it's name
+   * @param name - the name of stop to find
+   * @return the stop index or -1 if the stop is not found
+   */
+  int FindStopByName(const wxString &name) const;
+  unsigned GetCouplerCount() const { return m_couplers.size(); }
+  unsigned GetODFCouplerCount() const { return m_ODFCouplerCount; }
   GOCoupler *GetCoupler(unsigned index);
+  /**
+   * Find a coupler by it's name
+   * @param name - the name of coupler to find
+   * @return the stop index or -1 if the stop is not found
+   */
+  int FindCouplerByName(const wxString &name) const;
   void AddCoupler(GOCoupler *coupler);
   unsigned GetDivisionalCount();
   GODivisionalButtonControl *GetDivisional(unsigned index);
   void AddDivisional(GODivisionalButtonControl *divisional);
-  unsigned GetTremulantCount();
+  unsigned GetTremulantCount() const { return m_tremulant_ids.size(); }
   GOTremulant *GetTremulant(unsigned index);
-  unsigned GetSwitchCount();
+  /**
+   * Find a tremulant belonging to this manual by it's name
+   * @param name - the name of tremulant to find
+   * @return the tremulant index in the OrganModel (not in the manual) or -1
+   *   if the tremulant is not found
+   */
+  int FindTremulantByName(const wxString &name) const;
+  unsigned GetSwitchCount() const { return m_switch_ids.size(); }
   GOSwitch *GetSwitch(unsigned index);
+  /**
+   * Find a switch belonging to this manual by it's name
+   * @param name - the name of switch to find
+   * @return the switch index in the OrganModel (not in the manual) or -1
+   *   if the switch is not found
+   */
+  int FindSwitchByName(const wxString &name) const;
 
   GOCombinationDefinition &GetDivisionalTemplate();
   const wxString &GetName();
